#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 14:59
# @Author: Jtyoui@qq.com
from jtyoui.decorator import parameter_set_length

"""
平均值,中位数,众数,分位数,极差,方差,标准差,偏度,峰度
"""


class AnalysisMath:

    def __init__(self, data=()):
        if not isinstance(data, (set, list, tuple)):
            raise TypeError("传入一个可替代对象.比如list,set,tuple类型")
        self.__data = data
        self.__length = len(data)

    def average(self, flag=0):
        """平均数"""
        """
        flag=0 算术平均值
        flag=1 几何平均值
        flag=2 平方平均值(均方根)
        flag=3 调和平均值
        :param flag:根据不同的数值,进行不同的平均值
        """
        if flag == 0:
            return sum(self.__data) / self.__length
        elif flag == 1:
            all_ = 1
            for data in self.__data:
                all_ *= abs(data)
            return pow(all_, self.__length)
        elif flag == 2:
            all_ = [data ** 2 for data in self.__data]
            return pow(sum(all_) / self.__length, 0.5)
        elif flag == 3:
            all_ = [1 / data for data in self.__data]
            return self.__length / sum(all_)

    def expect(self, data=None):
        """数学期望"""
        if data:
            self.__init__(data)
        return self.average(0)

    @property
    def median(self):
        """中位数"""
        return self.quantile(2)

    @property
    def mode_number(self):
        """众数"""
        data = {}
        for d in self.__data:
            data[d] = data.get(d, 0) + 1
        max_ = sorted(data.items(), key=lambda x: x[1], reverse=True)
        return max_[0][0]

    def quantile(self, position=1):
        """分位数"""
        """
        position=1: 第一四分位数 (Q1)，又称“较小四分位数”，等于该样本中所有数值由小到大排列后第25%的数字。
        position=2: 第二四分位数 (Q2)，又称“中位数”，等于该样本中所有数值由小到大排列后第50%的数字。
        position=3: 第三四分位数 (Q3)，又称“较大四分位数”，等于该样本中所有数值由小到大排列后第75%的数字。
        :param position:根据不同的数值计算不同的分位数
        """
        if position > 3:
            raise ValueError("position的取值范围是[1,2,3]整数")

        data = sorted(self.__data)
        middle, mod = divmod(self.__length * position, 4)
        if mod == 0:
            return (data[middle - 1] + data[middle]) / 2
        else:
            return data[middle]

    @property
    def range(self):
        """极差"""
        return max(self.__data) - min(self.__data)

    def variance(self, data=None):
        """方差"""
        if data:
            self.__init__(data)
        average = self.average(0)
        all_ = [(data - average) ** 2 for data in self.__data]
        return sum(all_) / self.__length

    def standard(self, data=None):
        """标准差"""
        if data:
            self.__init__(data)
        return pow(self.variance(), 0.5)

    @property
    def skewness(self):
        """偏度(偏态系数)"""
        x_3 = [data ** 3 for data in self.__data]
        x_2 = [data ** 2 for data in self.__data]

        expect_1 = self.average(0)  # x的数学期望
        expect_3 = sum(x_3) / self.__length  # x**3的数学期望
        expect_2 = sum(x_2) / self.__length  # x**2的数学期望

        numerator = expect_3 - 3 * expect_1 * (expect_2 - expect_1 ** 2) - expect_1 ** 3  # 分子
        denominator = pow(expect_2 - expect_1 ** 2, 1.5)  # 分母
        return numerator / denominator

    @property
    def kurtosis(self):
        """峰度"""
        """
        如果超值峰度为正，称为尖峰态
        如果超值峰度为负，称为低峰态
        """
        average = self.average(0)  # 数学期望
        all_4 = [(data - average) ** 4 for data in self.__data]
        numerator = sum(all_4) / self.__length  # 四阶样本中心矩

        denominator = self.standard() ** 4  # 标准差**4

        return numerator / denominator - 3  # 通常将峰度值做减3处理，使得正态分布的峰度0


@parameter_set_length
def cov(x, y):
    """协方差"""
    """expect_x:表示x的数学期望"""
    if isinstance(x, (set, list, tuple)) and isinstance(y, (set, list, tuple)):
        ana = AnalysisMath()
        expect_x = ana.expect(x)
        expect_y = ana.expect(y)
        xy = [x_ * y_ for x_, y_ in zip(x, y)]
        expect_xy = ana.expect(xy)
        return expect_xy - expect_x * expect_y
    else:
        raise TypeError("x,y类型必须满足其中一个(set, list, tuple)")


if __name__ == '__main__':
    analysis = AnalysisMath(data=[1, 2, 3, 5, 7, 2, 4, 2, 8, 9])
    print(analysis.average(3))
    print(analysis.median)
    print(analysis.mode_number)
    print(analysis.quantile(2))
    print(analysis.range)
    print(analysis.variance())
    print(analysis.standard())
    print(analysis.skewness)
    print(analysis.kurtosis)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/13 16:23
# @Author: Jtyoui@qq.com
import json
import time
import requests


def interface_test(dict_, address, record_time=True, **kwargs):
    """简单接口测试
    :param dict_: 传入的参数
    :param address: url地址
    :param kwargs:参考requests.post **kwargs
    :param record_time: 是否记录消耗时间
    :return: 接口返回值
    """
    start = time.time()
    j = json.dumps(dict_, ensure_ascii='utf8')
    response = requests.post(address, j, **kwargs)
    page = json.loads(response.text)
    end = time.time()
    if record_time:
        return page, end - start
    return page


def interface_tests(method, url, data, record_time=True, **kwargs):
    """简单接口测试
    :param method: 请求方法，比如：post、get
    :param url: url地址
    :param data: 传入的参数
    :param record_time: 是否记录消耗时间
    :param kwargs: 参考requests.post **kwargs
    :return: 接口返回值
    """
    start = time.time()
    response = requests.request(method=method.upper(), url=url, data=data, **kwargs)
    end = time.time()
    if record_time:
        return response.text, end - start
    return response.text


interface_post_test = interface_test  # post接口测试
interface_all_tests = interface_tests  # 接口测试

if __name__ == '__main__':
    d = {'answer': '我要告南明区政府贪污。', 'event_id': 'df99f4bb7f94c69c1b37ece4b41f1d05'}
    ji = interface_test(d, 'http://222.85.147.140:10056/commit_org')
    print(ji)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/7/17 21:30
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from enum import Enum


class FileFormat(Enum):
    """文件格式介绍"""
    A = "对象代码库文件"
    AAM = "Author ware shocked文件"
    AAS = "Author ware shocked包"
    ABF = "Adobe二进制屏幕字体"
    ABK = "CorelDRAW自动备份文件"
    ABS = "该类文件有时用于指示一个摘要（就像在一篇有关科学方面的文章的一个摘要或概要，取自abstract）"
    ACE = "Ace压缩文件格式"
    ACM = "Windows系统目录文件"
    ACP = "Microsoft office助手预览文件"
    ACT = "Microsoft office助手文件"
    AD = "AfterDark屏幕保护程序"
    ADA = "Ada源文件（非-GNAT）"
    ADB = "Ada源文件主体（GNAT）；HP100LX组织者的约定数据库"
    ADF = "Amiga磁盘文件"
    ADI = "AutoCAD设备无关二进制绘图仪格式"
    ADM = "AfterDark多模块屏幕保护；WindowsNT策略模板"
    ADP = "FaxWork用于传真调制解调器的交互安装文件；AstoundDynamite文件"
    ADR = "AfterDark随机屏幕保护；SmartAddress的地址簿"
    ADS = "Ada源文件说明书（GNAT）"
    AFM = "Adobe的字体尺度"
    AF2 = "ABC的FlowChat文件"
    AF3 = "ABC的FlowChat文件"
    AI = "AdobeIllustrator格式图形"
    AIF = "音频互交换文件，SiliconGraphicandMacintosh应用程序的声音格式"
    AIFF = "音频互交换文件，SiliconGraphicandMacintosh应用程序的声音格式"
    AIFC = "压缩AIF"
    AIM = "AOL即时信息传送"
    AIS = "ACDSee图形序列文件；VelvetStudio设备文件"
    AKW = "RoboHELP的帮助工程中所有A-关键词"
    ALB = "JASCImageCommander相册"
    ALL = "艺术与书信库"
    AMS = "VelvetStudio音乐模块（MOD）文件；Extreme的Tracker模块文件"
    ANC = "CanonComputer的调色板文件，包含一系列可选的颜色板"
    ANI = "Windows系统中的动画光标"
    ANS = "ANSI文本文件"
    ANT = "SimAntForWindows中保存的游戏文件"
    API = "AdobeAcrobat使用的应用程序设计接口文件"
    APS = "MicrosoftVisualC++文件"
    APK = "安卓安装包"
    ARI = "Aristotle声音文件"
    ARJ = "RobertJungARJ压缩包文件"
    ART = "XaraStudio绘画文件；CanonCrayola美术文件；ClipArt文件格式；另一种光线跟踪格式；AOL使用的用Johnson-Grace压缩算法压缩的标记文件"
    ASA = "MicrosoftVisualInterDev文件"
    ASC = "ASCⅡ文本文件；PGP算法加密文件"
    ASD = "Microsoft Word的自动保存文件；Microsoft高级流媒体格式的描述文件；可用NSREX打开VelvetStudio例子文件"
    ASE = "VelvetStudio采样文件"
    ASF = "Microsoft高级流媒体格式文件"
    ASM = "汇编语言源文件，Pro/E装配文件"
    ASO = "AstoundDynamite对象文件"
    ASP = "动态网页文件；ProCommPlus安装与连接脚本文件；Astound介绍文件"
    ASV = "DataCAD自动保存文件"
    ASX = "Cheyenne备份脚本文件；Microsoft高级流媒体重定向器文件，视频文件"
    ATW = "来自个人软件的AnyTimeDeluxeForWindows个人信息管理员文件"
    AU = "Sun/NeXT/DEC/UNIX声音文件；音频U-Law（读作“mu-law”）文件格式"
    AVB = "ComputerAssociatesInoculan反病毒软件的病毒感染后文件"
    AVI = "MicrosoftAudioVideoInterleave电影格式"
    AVR = "AudioVisualResearch文件格式"
    AVS = "应用程序可视化格式"
    AWD = "FaxVien文档"
    AWR = "Telsis数字储存音频文件扩展名格式"
    Axx = "ARJ压缩文件的分包序号文件，用于将一个大文件压至几个小的压缩包中（xx取01-99的数字）"
    A3M = "Author ware Macintosh未打包文件"
    A4M = "Author ware Macintosh未打包文件"
    A4P = "Author ware无运行时间的打包文件"
    A3W = "未打包的Author ware Windows文件"
    A4W = "未打包的Author ware Windows文件"
    A5W = "未打包的Author ware Windows文件"
    BAK = "备份文件"
    BAS = "BASIC源文件"
    BAT = "批处理文件"
    BDF = "WestPointBridgerDesigner文件"
    BGL = "MicrosoftFlightSimulator（微软飞行模拟器）的视景文件"
    BI = "二进制文件"
    BIF = "GroupWise的初始化文件"
    BIFF = "XLIFE3D格式文件"
    BIN = "二进制文件"
    BK = "有时用于代表备份版本"
    BKS = "IBMBookManagerRead书架文件"
    BMK = "书签文件"
    BMP = "Windows或OS/2位图文件"
    BOOK = "AdobeFrameMakerBook文件"
    BRX = "用于查看多媒体对象目录的文件"
    BSP = "Quake图形文件"
    BTM = "Norton应用程序使用的批处理文件"
    BUD = "Quicken的备份磁盘"
    BUN = "CakeWalk声音捆绑文件（一种MIDI程序）"
    BW = "SGI黑白图像文件"
    BWV = "商业波形文件"
    C = "C代码文件"
    CAB = "Microsoft压缩文件"
    CAD = "Softdek的DrafixCAD文件"
    CAL = "CALS压缩位图；日历计划表数据"
    CAP = "压缩音乐文件格式"
    CAS = "逗号分开的ASCⅡ文件"
    CB = "Microsoft干净引导文件"
    CC = "VisualdBASE用户自定义类文件"
    CCB = "VisualBasic动态按钮配置文件"
    CCH = "Corel图表文件"
    CCO = "CyberChat数据文件"
    CCT = "MacromediaDirectorShockwave投影"
    CDA = "CD音频轨道"
    CDF = "Microsoft频道定义格式文件"
    CDI = "Philip的高密盘交互格式"
    CDM = "VisualdBASE自定义数据模块文件"
    CDR = "CorelDRAW绘图文件；原始音频CD数据文件"
    CDT = "CorelDRAW模板"
    CDX = "CorelDRAW压缩绘图文件；MicrosoftVisualFoxPro索引文件"
    CFG = "配置文件"
    CGI = "公共网关接口脚本文件"
    CGM = "计算机图形元文件"
    CHK = "由Windows磁盘碎片整理器或磁盘扫描保存的文件碎片"
    CHM = "编译过的HTML文件"
    CHR = "字符集（字体文件）"
    CIF = "AdaptecCD创建器CD映像文件"
    CIL = "ClipGallery下载包"
    CLASS = "Java类文件"
    CLL = "CrickSoftwareClicker文件"
    CLP = "Windows剪贴板文件"
    CLS = "VisualBasic类文件"
    CMF = "Corel元文件"
    CMV = "CorelMove动画文件"
    CMX = "CorelPresentationExchange图像"
    CNF = "Telnet，Windows和其他其内格式会发生改变的应用程序使用的配置文件"
    CNM = "Windows应用程序菜单选项和安装文件"
    CNT = "Windows（或其他）系统用于帮助索引或其他目的内容文件"
    COD = "MicrosoftC编译器产生的可显示机器码/汇编代码文件，其中附有源C代码作为注释"
    COM = "命令文件（程序）"
    CPL = "控制面板扩展名，Corel颜色板"
    CPO = "Corel打印存储文件"
    CPP = "C++代码文件"
    CPR = "Corel提供说明书文件"
    CPT = "Corel照片-绘画图像"
    CPX = "CorelPresentationExchange压缩图形文件"
    CRD = "WindowsCardfile文件"
    CRP = "Corel提供的运行时介绍文件；VisualdBASE自定义报表文件"
    CRT = "认证文件"
    CSC = "Corel脚本文件"
    CSP = "PCEmceeOn_Screen图像"
    CST = "MacromediaDirectorCast文件"
    CSV = "逗号分隔的值文件"
    CTL = "通常用于表示一个包含控件信息的文件；FaxWork用它来保持有关每个传真收到或发出的信息"
    CUR = "Windows光标文件"
    CV = "Corel版本的档案文件；MicrosoftCodeView信息屏幕文件"
    CXX = "C++源代码文件"
    DAT = "数据文件；WrodPerfect合并数据文件；用于一些MPEG格式的文件"
    DB = "Borland的Paradox7表"
    DBC = "MicrosoftVisualFoxPro数据库容器文件"
    DBF = "dBASE文件"
    DBX = "DataBearn图像；MicrosoftVisualFoxPro表格文件"
    DCM = "DCM模块格式文件"
    DCS = "桌面颜色分隔文件"
    DCT = "MicrosoftVisualFoxPro数据库容器"
    DCU = "Delphi编译单元文件"
    DCX = "MicrosoftVisualFoxPro数据库容器；基于PCX的传真图像；宏"
    DEM = "用于表示数字高度模型的USGS基准的文件"
    DER = "认证文件"
    DEWF = "MacintoshSoundCap/SoundEdit录音设备格式"
    DIB = "设备无关位图"
    DIC = "目录"
    DIF = "可进行数据互换的电子表格"
    DIG = "DigiLink格式；SoundDesignerⅠ音频文件"
    DIR = "MacromediaDirector文件"
    DIZ = "描述文件"
    DLG = "C++对话框脚本文件"
    DLL = "动态链接库"
    DLS = "可下载声音文件"
    DMD = "VisualdBASE数据模块文件"
    DMF = "X-Trakker音乐模块（MOD）文件"
    DOC = "Word文档"
    DOT = "MicrosoftWord文档模板"
    DRAW = "Acorn的基于对象的矢量图像文件"
    DRV = "驱动程序"
    DRW = "MicrografxDesigner/Draw；Pro/E绘画文件"
    DSF = "MicrografxDesignerVFX文件"
    DSG = "DOOM保存的文件"
    DSM = "DynamicStudio音乐模块（MOD）文件"
    DSP = "MicrosoftDeveloperStudio工程文件"
    DSQ = "CorelQUERY（查询）文件"
    DSW = "MicrosoftDeveloperStudio工作区文件"
    DTD = "SGML文档类型定义（DTD）文件"
    DUN = "Microsoft拔号网络导出文件"
    DV = "数字视频文件（MIME）"
    DXF = "可进行互交换的绘图文件格式，二进制的DWG格式的文本表示；数据交换文件"
    DXR = "MacromediaDirector受保护（不可编辑）电影文件"
    EDA = "EnsoniqASR磁盘映像"
    EDD = "元素定义文档（FrameMaker+SGML文档）"
    EMD = "ABT扩展模块"
    EMF = "Windows增强元文件"
    EML = "MicrosoftOutlookExpress邮件消息（MIMERFC822）文件"
    EPHTML = "Perl解释增强HTML文件"
    EPS = "压缩的PostScript图像"
    EXE = "可执行文件（程序）"
    FAV = "MicrosoftOutlook导航条"
    FAX = "传真类型图像"
    FCD = "虚拟CD-ROM"
    FDF = "AdobeAcrobat表单文档文件"
    FFA = "Microsoft快速查找文件"
    FFL = "Microsoft快速查找文件"
    FFO = "Microsoft快速查找文件"
    FFK = "Microsoft快速查找文件"
    FIF = "Fractal图像文件"
    FLA = "MacromediaFlash电影"
    FLC = "AutodeskFLIC动画文件"
    PLI = "AutodeskFLIC动画"
    FM = "AdobeFrameMaker文档"
    FML = "文件镜象列表（GetRight）"
    FNG = "字体组文件（字体导航器，FontNavigator）"
    FNK = "FunkTracker模块格式"
    FON = "系统字体"
    FOT = "字体相关文件"
    FRT = "MicrosoftFoxPro报表文件"
    FRX = "VisualBasic表单文本；MicrosoftFoxPro报表文件"
    FTG = "全文本搜索组文件，由Windows帮助系统查找时产生--可以删除，并在需要时重建起来"
    FTS = "全文本搜索引文件，由Windows帮助系统查找时产生"
    GAL = "Corel多媒体管理器相集"
    GDB = "InterBase数据库文件"
    GDM = "铃声、口哨声和声音板模块格式"
    GEM = "GEM元文件"
    GEN = "Ventura产生的文本文件"
    GetRight = "GetRight未完成的下载文件"
    GFI = "Genigraphics图形链接表示文件"
    GFX = "Genigraphics图形链接表示文件"
    GHO = "Norton克隆磁盘映像"
    GIF = "CompuServe位图文件"
    GIM = "Genigraphics图形链接介绍文件"
    GIX = "Genigraphics图形链接介绍文件"
    GKH = "EnsoniqEPS家簇磁盘映像文件"
    GKS = "GravisGripKey文档"
    GL = "动画格式"
    GNA = "Genigraphics图形链接介绍文件"
    GNT = "生成代码，MicroFocus属性格式里的可执行代码"
    GNX = "Genigraphics图形链接介绍文件"
    GRA = "MicrosoftGraph文件"
    GRF = "Grapher（GoldenSoftware公司）图形文件"
    GRP = "程序管理组"
    GO = "Go语言源文件"
    HCOM = "声音工具HCOM格式"
    HGL = "HP图形语言绘图文件"
    HLP = "帮助文件；DateCADWindows帮助文件"
    HPJ = "VisualBasic帮助工程"
    HPP = "C++程序头文件"
    HST = "历史文件"
    HT = "HyperTerminal（超级终端）"
    HTM = "超文本文档"
    HTML = "超文本文档"
    HTT = "Microsoft超文本模板"
    HTX = "扩展HTML模板"
    ICB = "Targa位图文件"
    ICC = "Kodak打印机格式文件"
    ICL = "图标库文件"
    ICM = "图形颜色匹配配置文件"
    ICO = "Windows图标"
    IDD = "MIDI设备定义"
    IDF = "MIDI设备定义（Windows95需要的文件）"
    IDQ = "Internet数据查询文件"
    IDX = "MicrosoftFoxPro相关数据库索引文件；SymantecQ&A相关数据库索引文件；MicrosoftOutlookExpress文件"
    IFF = "交换格式文件；AmigaILBM"
    IGES = "初始图形交换说明文件"
    IGF = "插入系统元文件"
    ILBM = "位图图形文件"
    IMA = "WinImage磁盘映像文件"
    INF = "信息文件"
    INI = "初始化文件"
    INRS = "INRS远程通信声频"
    INS = "InstallShield安装脚本"
    INT = "中间代码，当一个源程序经过语法检查后编译产生一个可执行代码"
    IQY = "MicrosoftInternet查询文件"
    ISO = "根据ISD9660有关CD-ROM文件系统标准列出CD-ROM上的文件"
    IST = "数字跟踪设备文件"
    ISU = "InstallShield卸装脚本"
    IWC = "InstallWatch文档"
    J62 = "Ricoh照相机格式"
    JAR = "Java档案文件"
    JAVA = "Java源文件"
    JBF = "PaintShopPro图像浏览文件"
    JFF = "JPEG文件"
    JFIF = "JPEG文件"
    JIF = "JPEG文件"
    JMP = "SAS的JMPDiscovery表格统计文件"
    JPE = "JPEG图形文件"
    JPEG = "JPEG图形文件"
    JPG = "JPEG图形文件"
    JS = "Javascript源文件"
    JSP = "HTML网页，其中包含有对一个Javaservlet的参考"
    JSON = "Json文件"
    JTF = "JPEG位图文件"
    K25 = "Kurzweil2500抽样文件"
    KAR = "卡拉OKMIDI文件"
    KDC = "Kodak光增强器"
    KEY = "DataCAD图标工具条文件"
    KFX = "KoFakGroup4图像文件"
    KIZ = "Kodak数字明信片文件"
    KKW = "RoboHELP帮助工程索引设计器中与主题无关的K开头的所有关键字"
    KMP = "KorgTrinityKeyMap文件"
    KQP = "Konica照相机本地文件"
    LAB = "VisualdBASE标签文件"
    LBM = "DeluxePaint位图文件"
    LBT = "MicrosoftFoxPro标签文件"
    LBX = "MicrosoftFoxPro标签文件"
    LDB = "MicrosoftAccess加锁文件"
    LDL = "CorelParadox分发库"
    LEG = "Legacy文档"
    LFT = "3DStudio（DOS）放样文件"
    LGO = "Paintbrush（Microsoft画图应用程序）的徽标文件"
    LHA = "LZH更换文件后缀"
    LIB = "库文件"
    LIN = "DataCAD线型文件"
    LIS = "结构化查询报告（SQR）程序产生的输出文件"
    LLX = "Laplink交换代理"
    LNK = "Windows快捷方式文件"
    LOG = "日志文件"
    LST = "列表文件"
    LU = "ThoughtWing库单元文件"
    LYR = "DataCAD层文件"
    LZH = "LHARC压缩档案"
    LZS = "Skyroads数据文件"
    M1V = "MPEG相关文件（"
    M3U = "MPEGURL（MIME声音文件）"
    MAD = "MicrosoftAccess模块文件"
    MAF = "MicrosoftAccess表单文件"
    MAM = "MicrosoftAccess宏"
    MAP = "映射文件；DukeNukem3DWAD游戏文件"
    MAQ = "MicrosoftAccess查询文件"
    MAR = "MicrosoftAccess报表文件"
    MAT = "MicrosoftAccess表；3DStudioMAX材料库"
    MB1 = "ApogeeMonsterBash数据文件"
    MBX = "MicrosoftOutlook保存email格式；Eudora邮箱"
    MCR = "DataCAD键盘宏文件"
    MDB = "MicrosoftAccess数据库"
    MDE = "MicrosoftAccessMDE文件"
    MDL = "数字跟踪器音乐模块（MOD）文件；Quake模块文件"
    MDN = "MicrosoftAccess空数据库模板"
    MDW = "MicrosoftAccess工作组文件"
    MDZ = "MicrosoftAccess向导模板文件"
    MIC = "MicrosoftImageComposer文件"
    MID = "MIDI音乐"
    MIM = "MIME，MMEInternet邮件扩展格式的多用途文件，经常作为发送e-mail时在AOL里附件而创建的文件；在一个多区MIM文件里的文件能用WinZip或其他类似程序打开"
    MLI = "3DStudio的材料库格式文件"
    MNG = "多映像网络图形"
    MNU = "VisualdBASE菜单文件；IntertelSystemsInteract菜单文件"
    MOD = "FastTracker、StarTrekker、NoiseTracker（等等）音乐模块文件；Microsoft多计划电子表格；Amiga/PC磁道文件"
    MOV = "QuickTimeforWindows电影"
    MP2 = "第二层MPEG音频文件"
    MP3 = "第三层MPEG音频文件"
    MPA = "MPEG相关文件，MIME“mpeg类型”"
    MPE = "MPEG动画文件"
    MPEG = "MPEG动画文件"
    MPG = "MPEG动画文件"
    MPP = "Microsoft工程文件；CAD绘图文件格式"
    MPR = "MicrosoftFoxPro菜单（已编译）"
    MSG = "Microsoft邮件消息"
    MSI = "Windows安装文件包"
    MSN = "Microsoft网络文档；DescentMission文件"
    MSP = "MicrosoftPaint（画图）位图文件；WindowsInstaller路径文件"
    MST = "Windows安装器传输文件"
    MTM = "Multi跟踪器音乐模块（MOD）文件"
    NAN = "Nanoscope文件（RawGrayscale）"
    NAP = "NAP元文件"
    NCB = "MicrosoftDeveloperStudio文件"
    NCD = "Norton改变目录"
    NCF = "NetWare命令文件；LotusNotes内部剪切板"
    NFF = "中性文件格式"
    NFT = "NetObjectFusion模板文件"
    NIL = "Norton光标库文件（EasyIcons-兼容）"
    NIST = "NISTSphere声音"
    NLS = "用于本地化的国家语言支持文件（例如，Uniscape）"
    NLU = "NortonLiveUpdatee-mail触发器文件"
    NTX = "CA-Clipper索引文件"
    NWC = "NoteworthyComposer歌曲文件"
    NWS = "MicrosoftOutlookExpress新闻消息"
    OBJ = "对象文件"
    OCX = "Microsoft对象链接与嵌入定制控件"
    ODS = "MicrosoftOutlookExpress邮箱文件"
    OFN = "MicrosoftOfficeFileNew文件"
    OFT = "MicrosoftOutlook模板"
    OLB = "OLE对象库"
    OLE = "OLE对象"
    OOGL = "面向对象图形库"
    OPO = "OPL输出可执行文件"
    P65 = "PageMaker6.5文件"
    PAB = "Microsoft个人地址簿"
    PART = "Zilla部分下载文件"
    PAS = "Pascal源代码"
    PY = "Python源代码"
    PYC = "Python字节码文件"
    PYW = "Python图形窗口文件"
    PBD = "PowerBuilder动态库，作为本地DLL的一个替代物"
    PBL = "用于在PowerBuilder开发环境中的PowerBuilder动态库"
    PBM = "可导出位图"
    PBR = "PowerBuilder资源"
    PCD = "KodakPhoto-CD映像；P-Code编译器测试脚本，由Microsoft测试与MicrosoftVisual测试"
    PCL = "Hewlett-Packard打印机控制语言文件（打印机备用位图）"
    PCM = "声音文件格式；OKIMSM6376合成芯片PCM格式"
    PDD = "可以用PaintShopPro或其他图像处理软件打开的图形图像"
    PDF = "AdobeAcrobat可导出文档格式文件（可用Web浏览器显示）；Microsoft系统管理服务器包定义文件；NetWare打印机定义文件"
    PFM = "打印机字体尺度"
    PGL = "HP绘图仪绘图文件"
    PGM = "可输出灰度图（位图）"
    PH = "由Microsoft帮助文件编译器产生的临时文件"
    PHP = "包含有PHP脚本的HTML网页"
    PHP3 = "包含有PHP脚本的HTML网页"
    PHTML = "包含有PHP脚本的HTML网页；由Perl分析解释的HTML"
    PIC = "PC画图位图；Lotus图片；MacintoshPICT绘图"
    PJX = "MicrosoftVisualFoxPro工程文件"
    PJT = "MicrosoftVisualFoxPro工程文件"
    PKG = "MicrosoftDeveloperStudio应用程序扩展（与DLL文件类似）"
    PNG = "可移植的网络图形位图；PaintShopPro浏览器目录"
    POT = "MicrosoftPowerpoint模块"
    PPA = "MicrosoftPowerpoint内插器"
    PPF = "TurtleBeach的Pinnacle程序文件"
    PPM = "可移植的象素映射位图"
    PPS = "MicrosoftPowerpoint幻灯片放映"
    PPT = "MicrosoftPowerpoint演示文稿"
    PRF = "Windows系统文件，Macromedia导演设置文件"
    PRG = "dBASEClipper和FoxPro程序源文件；WAVmaker程序"
    PRJ = "3DStudio（DOS）工程文件"
    PRN = "打印表格（用空格分隔的文本）；DataCADWindows打印机文件"
    PRT = "打印格式化文件；Pro/ENGINEER元件文件"
    PSD = "Adobephotoshop位图文件"
    PSP = "PaintShopPro图像文件"
    PST = "MicrosoftOutlook个人文件夹文件"
    PWZ = "MicrosoftPowerpoint向导"
    QIC = "Microsoft备份文件"
    QIF = "QuickTime相关图像（MIME）；Quicken导入文件"
    QLB = "Quick库"
    QRY = "Microsoft查询文件"
    QTP = "QuickTime优先文件"
    QTX = "QuickTime相关图像"
    QW = "SymantecQ&AWrite程序文件"
    RA = "RealAudio声音文件"
    RAM = "RealAudio元文件"
    RAR = "RAR压缩档案（EugeneRoshall格式）"
    RDF = "资源描述框架文件（涉及XML和元数据）"
    REG = "注册表文件"
    REP = "VisualdBASE报表文件"
    RES = "MicrosoftVisualC++资源文件"
    RFT = "可修订的表单文本（IBM的DCA一部分或文档内容框架结构一部分）"
    RGB = "Silicon图形RGB文件"
    SGI = "Silicon图形RGB文件"
    RM = "RealAudio视频文件"
    RMD = "MicrosoftRegMaid文档"
    RPT = "MicrosoftVisualBasicCrystal报表文件"
    RTF = "RichText格式文档"
    RUL = "InstallShield使用的扩展名"
    RVP = "MicrosoftScan配置文件（MIME）"
    S = "汇编源代码文件"
    SAV = "游戏保存文件"
    SBL = "ShockwaveFlash对象文件"
    SCC = "MicrosoftSourceSafe文件"
    SCF = "WindowsExplorer命令文件"
    SCP = "拨号网络脚本文件"
    SCR = "Windows屏幕保护；传真图像；脚本文件"
    SCT = "SAS目录（DOS）；ScitexCT位图；MicrosoftFoxPro表单"
    SCX = "MicrosoftFoxPro表单文件"
    SDT = "SmartDraw模板"
    SDV = "分号分隔的值文件"
    SDX = "由SDX压缩的MIDI抽样转储标准文件"
    SEP = "标签图像文件格式（TIFF）位图"
    SFD = "SoundStage声音文件数据"
    SFI = "SoundStage声音文件信息"
    SFR = "SonicFoundrySample资源"
    SFX = "RAR自解压文件"
    SGML = "标准通用标签语言"
    SHG = "热点位图"
    SHTML = "含有服务器端包括（SSI）的HTML文件"
    SHW = "CorelShow演示文稿"
    SIG = "符号文件"
    SKA = "PGP秘钥"
    SKL = "Macromedia导演者资源文件"
    SL = "PACT的保存布局扩展名"
    SO = "Linux的共享库"
    SPL = "ShockwaveFlash对象；DigiTrakker抽样"
    SQC = "结构化查询语言（SQR）普通代码文件"
    SQR = "结构化查询语言（SQR）程序文件"
    STR = "屏幕保护文件"
    SWA = "在Macromedia导演文件（MP3文件）中的Shockwave声音文件"
    SWF = "ShockwaveFlash对象"
    SYS = "系统文件"
    SYW = "YamahaSY系列波形文件"
    TAZ = "UNIXgzip/tape档案"
    TGA = "Targa位图"
    THEME = "Windows桌面主题文件"
    THN = "GraphicsWorkShopforWindows速写"
    TIF = "标签图像文件格式（TIFF）位图"
    TIFF = "标签图像文件格式（TIFF）位图"
    TIG = "虎形文件，美国政府用于分发地图"
    TLB = "OLE类型库"
    TMP = "Windows临时文件"
    TOL = "Kodak照片增强器"
    TPL = "CakeWalk声音模板文件；DataCAD模板文件"
    TRM = "终端文件"
    TRN = "MKS源完整性工程用法日志文件"
    TTF = "TrueType字体文件"
    TXT = "ASCⅡ文本格式的声音数据"
    TXW = "YamahaTX16W波形文件"
    UDF = "WindowsNT/2000唯一性数据库文件"
    ULT = "UltraTracker音乐模块（MOD）文件"
    URL = "Internet快捷方式文件"
    USE = "MKS源完整性文件"
    UWF = "Ultraracker波形文件"
    VBP = "MicrosoftVisualBasic工程文件"
    VBS = "MicrosoftVisualBasic脚本语言"
    VBW = "MicrosoftVisualBasic工作区文件"
    VBX = "MicrosoftVisualBasic用户定制控件"
    VCT = "MicrosoftFoxPro类库"
    VCX = "MicrosoftFoxPro类库"
    VDA = "Targa位图"
    VIR = "NortonAnti-Virus或其他杀毒产品用于标识被病毒感染的文件"
    VIV = "VivoActivePlayer流视频文件"
    VSD = "Visio绘画文件（流程图或图解）"
    VSL = "下载列表文件（GetRight）"
    VSS = "Visio模板文件"
    VST = "Targa位图"
    VSW = "Visio工作区文件"
    VXD = "MicrosoftWindows虚拟设备驱动程序"
    VQF = "YamahaSound-VQ文件（可能出现标准）"
    W3L = "W3Launch文件"
    WAB = "MicrosoftOutlook文件"
    WAD = "包含有视频、玩家水平和其他信息的DOOM游戏的大文件"
    WAV = "Windows波形声形"
    WBK = "MicrosoftWord备份文件"
    WCM = "WordPerfect宏"
    WDB = "MicrosoftWorks数据库"
    WFM = "VisualdBASEWindows表单"
    WFN = "在CorelDRAW中使用的符号"
    WIL = "WinImage文件"
    WIZ = "MicrosoftWord向导"
    WLL = "MicrosoftWord内插器"
    WMF = "Windows元文件"
    WOW = "GraveComposer音乐模块（MOD）文件"
    WP = "WordPerfect文档"
    WPD = "WordPerfect文档或演示"
    WPF = "可字处理文档"
    WPG = "WordPerfect图形"
    WPS = "MicrosoftWorks文档"
    WPT = "WordPerfect模板"
    WR1 = "书写器文档"
    WRK = "Cakewalk音乐声音工程文件"
    WRL = "虚拟现实模型"
    WRZ = "VRML文件对象"
    X = "AVS图像格式"
    XAR = "CorelXARA绘画"
    XBM = "MIME“xbitmap”图像"
    XI = "ScreamTracker设备抽样文件"
    XLA = "MicrosoftExcel内插器"
    XLB = "MicrosoftExcel工具条"
    XLC = "MicrosoftExcel图表"
    XLD = "MicrosoftExcel对话框"
    XLK = "MicrosoftExcel备份"
    XLL = "MicrosoftExcel内插器文件"
    XLM = "MicrosoftExcel宏"
    XLS = "MicrosoftExcel工作单"
    XLT = "MicrosoftExcel模板"
    XLV = "MicrosoftExcelVBA模块"
    XLW = "MicrosoftExcel工作簿/工作区"
    XNK = "MicrosoftExchange快捷方式文件"
    XPM = "X位图格式"
    XWD = "XWindows转储格式"
    XWF = "YamahaXGWorks文件（MIDI序列）"
    X16 = "宏媒体扩展（程序扩展），16位"
    X32 = "宏媒体扩展（程序扩展），32位"
    YAL = "Arts&Letters剪贴艺术库"
    Z = "UNIX gzip文件"
    ZAP = "Windows软件安装配置文件"
    ZIP = "Zip文件"


if __name__ == '__main__':
    print(FileFormat.FFA.name, FileFormat.FFA.value)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/26 18:03
# @Author: Jtyoui@qq.com
from jtyoui.web import get


def mention_to_entity(mention):
    """输入名称->实体
    :param mention: 输入名称
    :return: 根据名称获取歧义关系
    """
    url = f'https://api.ownthink.com/kg/ambiguous?mention={mention}'  # 知识图谱API，歧义关系
    return get(url).text


def entity_to_knowledge(entity):
    """ 实体->知识
    :param entity: 实体名
    :return: 根据实体获取实体知识
    """
    url = f'https://api.ownthink.com/kg/knowledge?entity={entity}'  # 知识图谱API，实体知识
    return get(url).text


def entity_attribute_value(entity, attribute):
    """实体&属性->属性值
    :param entity: 实体名
    :param attribute: 属性名
    :return: 根据实体、属性获取属性值
    """
    url = f'https://api.ownthink.com/kg/eav?entity={entity}&attribute={attribute}'  # 知识图谱API，属性值
    return get(url).text


if __name__ == '__main__':
    print(mention_to_entity('红楼梦'))

    print(entity_to_knowledge('刘德华'))

    print(entity_attribute_value('苹果', '颜色'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/7 10:14
# @Author: Jtyoui@qq.com
"""这个包都是一些机器学习的用例算法，不提供使用！"""
from .common import *  # 机器学习常见的函数
from .testdata import *  # 训练和测试数据集合
from .neighbor import *  # 增加KNN算法



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/17 14:43
# @Author: Jtyoui@qq.com
from jtyoui.error import NotLegitimateNumberError
from jtyoui.file_zip import load_zip
from functools import reduce
import math


def real_number(value):
    """判断是否是实数"""
    if isinstance(value, (int, float)):
        return value
    elif isinstance(value, str):
        return float(value)
    elif isinstance(value, complex):
        raise TypeError('不支持复数')
    else:
        raise ValueError('类型错误')


def sign_function(value):
    """符号函数"""
    value = real_number(value)
    if value < 0:
        return -1
    elif value > 0:
        return 1
    return 0


def abs_function(value):
    """绝对值函数"""
    value = real_number(value)
    return abs(value)


def integral_function(value):
    """取整函数"""
    value = real_number(value)
    return value // 1


def dirichlet_function(value):
    """狄利克雷函数：只是简单的判断"""
    value = real_number(value)
    a, b = value.as_integer_ratio()
    if len(str(a)) >= 15 and len(str(b)) >= 15:
        return 0
    return 1


def is_prime(n):
    """判断一个数是否为质数"""
    if (not isinstance(n, int)) or (n <= 1):
        raise NotLegitimateNumberError('不是一个合法的数字')
    if n % 2 == 0:  # 判断偶数
        return False
    else:
        for i in range(3, int(math.sqrt(n)) + 1, 2):  # 判断奇数
            if n % i == 0:
                return False
        return True


def primes(n):
    """埃拉托斯特尼质数筛法
    >>> print(len(list(primes(1_0000_0000))))  # 时间5.5541136264801025秒
     10万内：9592
     100万内：78498
     1000万内：664579
     1亿内：5761455
    """
    n += 1
    ps = [True] * n
    total = [2]
    half = int(math.sqrt(n))
    for index in range(3, half + 1, 2):
        if is_prime(index):  # 取开平方的质数
            ps[index * index: n: index] = [False] * math.ceil((n - index * index) / index)
    for y in range(3, n, 2):
        if ps[y]:
            total.append(y)
    return total


def collatz(n):
    """Collatz猜想:也叫3n+1猜想，给一个正整数，如果是偶数，则减半；如果是奇数，则变为它的三倍加一。直到变为一停止"""
    if n > 1 and isinstance(n, int):
        total = 0
        while n != 1:
            if n % 2 == 0:
                n /= 2
            else:
                n = 3 * n + 1
            total += 1
    else:
        raise TypeError('类型错误。必须是大于1的整数')
    return total


def tetrahedron_volume(r1, r2, r3, r4, r5, r6):
    """知道四面体的边求体积，r1-r6都是边"""
    R1, R2, R3, R4, R5, R6 = r1 ** 2, r2 ** 2, r3 ** 2, r4 ** 2, r5 ** 2, r6 ** 2
    one = R1 * R2 * R4 + R1 * R3 * R6 + R2 * R3 * R5 + R4 * R5 * R6
    two = R1 * R2 * R5 + R1 * R2 * R6 + R1 * R3 * R4 + R1 * R3 * R5 + R1 * R4 * R5 + R1 * R5 * R6 + R2 * R3 * R4 + R2 * R3 * R6 + R2 * R4 * R6 + R2 * R5 * R6 + R3 * R4 * R5 + R3 * R4 * R6
    three = R1 * R1 * R5 + R1 * R5 * R5 + R2 * R2 * R6 + R2 * R6 * R6 + R3 * R3 * R4 + R3 * R4 * R4
    s = two - one - three
    v = math.sqrt(s / 144)
    return v


def tetrahedron_volume2(a, b, c, m, n, l):
    """知道四面体的边求体积，a, b, c, m, n, l都是边"""
    v = math.sqrt((4.0 * a * a * b * b * c * c - a * a * (b * b + c * c - m * m) * (b * b + c * c - m * m) - b * b * (
            c * c + a * a - n * n) * (c * c + a * a - n * n) - c * c * (a * a + b * b - l * l) * (
                           a * a + b * b - l * l) + (a * a + b * b - l * l) * (b * b + c * c - m * m) * (
                           c * c + a * a - n * n))) / 12.0
    return v


def helen_formula(a, b, c):
    """海伦公式，知道三边求面积
    a、b、c是三角形的三条边
    """
    if a > 0 and b > 0 and c > 0:
        p = (a + b + c) / 2
        s = math.sqrt(p * (p - a) * (p - b) * (p - c))
        return s
    else:
        raise TypeError('类型错误,三角形三边必须是大于0的数字类型')


def factorial(n, number=1):
    """求n的阶乘
    当number==1时，表示n!
    当number==2时，表示n!!
    当number==3时，表示n!!!
    ...................
    :param n: 输入大于1的整数
    :param number: 阶乘数
    :return: n的number阶乘
    """
    return reduce(lambda x, y: x * y, range(1, n + 1, number))


def pi(n=7):
    """计算PI,能精确到小数点：15万5千6百42位
    :param n:表示精确的小数，n的范围是：0-155640
    :return:返回的是字符串，默认是返回小数点7位
    """
    line = load_zip('pi.zip', 'pi.txt')
    return line[0][:n + 2]


if __name__ == '__main__':
    print(sign_function(123.22))
    print(integral_function(25.2))
    print(integral_function('-3.1'))
    print(dirichlet_function(math.pi))
    print(is_prime(915452))
    # print(len(primes(1_0000_0000)))  # 时间5.5541136264801025秒
    print(collatz(27))
    print(helen_formula(3, 3, 3))
    print(tetrahedron_volume(3, 3, 3, 3, 3, 3))
    print(tetrahedron_volume2(3, 3, 3, 3, 3 * math.sqrt(3), 3 * math.sqrt(3)))
    print(factorial(5, 2))
    print(pi())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/11 11:44
# @Author: Jtyoui@qq.com
from jtyoui.neuralNetwork.paddle.ernie.transformer_encoder import encoder, pre_process_layer
from jtyoui.neuralNetwork.paddle.ernie.vocab import vocal
import os
import numpy as np
from paddle import fluid  # pip install paddlepaddle==1.6.1

ERNIE_MODEL_PARAMETER = {
    "attention_probs_dropout_prob": 0.1,
    "hidden_act": "relu",
    "hidden_dropout_prob": 0.1,
    "hidden_size": 768,
    "initializer_range": 0.02,
    "max_position_embeddings": 513,
    "num_attention_heads": 12,
    "num_hidden_layers": 12,
    "type_vocab_size": 2,
    "vocab_size": 18000
}
ERNIE_LABEL_MAP = {
    "B-PER": 0,  # 人名
    "I-PER": 1,
    "B-ORG": 2,  # 机构名
    "I-ORG": 3,
    "B-LOC": 4,  # 地名
    "I-LOC": 5,
    "O": 6
}

# 需要自己更改
model_path, config, label_map_config = None, ERNIE_MODEL_PARAMETER, ERNIE_LABEL_MAP


def pad_batch_data(inst, pad_idx=0, input_mask=False):
    return_list = []
    max_len = max(len(inst) for inst in inst)
    inst_data = np.array([inst + list([pad_idx] * (max_len - len(inst))) for inst in inst])
    return_list += [inst_data.astype("int64").reshape([-1, max_len, 1])]
    if input_mask:
        input_mask_data = np.array([[1] * len(inst) + [0] * (max_len - len(inst)) for inst in inst])
        input_mask_data = np.expand_dims(input_mask_data, axis=-1)
        return_list += [input_mask_data.astype("float32")]
    return return_list if len(return_list) > 1 else return_list[0]


def prepare_batch_data(example):
    example = example.lower()
    words = [1] + [vocal[word] for word in example if word in vocal] + [2]
    padded_token_ids, input_mask = pad_batch_data([words], 0, True)
    padded_text_type_ids = pad_batch_data([[0] * len(words)])
    padded_position_ids = pad_batch_data([list(range(len(words)))])
    padded_label_ids = pad_batch_data([[8] * len(words)], len(label_map_config) - 1)
    return_list = [padded_token_ids, padded_text_type_ids, padded_position_ids, input_mask, padded_label_ids]
    yield return_list


def data_generator(input_str):
    def wrapper():
        for batch_data in prepare_batch_data(input_str):
            yield batch_data

    return wrapper


def init_checkpoint(exe, init_checkpoint_path, main_program):
    def existed(var):
        if not fluid.io.is_persistable(var):
            return False
        return os.path.exists(os.path.join(init_checkpoint_path, var.name))

    fluid.io.load_vars(exe, init_checkpoint_path, main_program=main_program, predicate=existed)


def evaluate(exe, program, reader, graph_vars):
    fetch_list = [graph_vars["labels"].name, graph_vars["infers"].name]
    total_number = None
    while True:
        reader.start()
        try:
            _, np_infers = exe.run(program=program, fetch_list=fetch_list)
            total_number = [ls[0] for ls in np_infers[1:-1]]
        except Exception as e:
            assert 'There is no next data' in e.__str__(), Exception('非正常停止。')
            reader.reset()
            break
    return total_number


def create_model():
    reader = fluid.layers.py_reader(capacity=50, shapes=[[-1, 256, 1]] * 5, lod_levels=[0] * 5, use_double_buffer=True,
                                    dtypes=['int64'] * 3 + ['float32', 'int64'])

    src_ids, sent_ids, pos_ids, input_mask, labels = fluid.layers.read_file(reader)
    self_attn_mask = fluid.layers.matmul(x=input_mask, y=input_mask, transpose_y=True)
    self_attn_mask = fluid.layers.scale(x=self_attn_mask, scale=10000.0, bias=-1.0, bias_after_scale=False)
    n_head_self_attn_mask = fluid.layers.stack(x=[self_attn_mask] * config['num_attention_heads'], axis=1)
    n_head_self_attn_mask.stop_gradient = True
    param_initializer = fluid.initializer.TruncatedNormal(config['initializer_range'])
    emb_out = fluid.layers.embedding(
        input=src_ids,
        size=[config['vocab_size'], config['hidden_size']],
        dtype="float32",
        param_attr=fluid.ParamAttr(name="word_embedding", initializer=param_initializer), is_sparse=False)

    position_emb_out = fluid.layers.embedding(
        input=pos_ids,
        size=[config['max_position_embeddings'], config['hidden_size']],
        dtype="float32",
        param_attr=fluid.ParamAttr(name="pos_embedding", initializer=param_initializer))

    sent_emb_out = fluid.layers.embedding(
        sent_ids,
        size=[config['type_vocab_size'], config['hidden_size']],
        dtype="float32",
        param_attr=fluid.ParamAttr(name="sent_embedding", initializer=param_initializer))

    emb_out += position_emb_out + sent_emb_out
    emb_out = pre_process_layer(emb_out, 'nd', config['hidden_dropout_prob'], name='pre_encoder')

    enc_out = encoder(
        n_layer=config['num_hidden_layers'],
        enc_input=emb_out,
        attn_bias=n_head_self_attn_mask,
        n_head=config['num_attention_heads'],
        d_key=config['hidden_size'] // config['num_attention_heads'],
        d_value=config['hidden_size'] // config['num_attention_heads'],
        d_model=config['hidden_size'],
        d_inner_hid=config['hidden_size'] * 4,
        prepostprocess_dropout=config['hidden_dropout_prob'],
        attention_dropout=config['attention_probs_dropout_prob'],
        relu_dropout=0,
        hidden_act=config['hidden_act'],
        preprocess_cmd="",
        postprocess_cmd="dan",
        param_initializer=param_initializer,
        name='encoder')

    log = fluid.layers.fc(input=enc_out, size=len(label_map_config), num_flatten_dims=2,
                          param_attr=fluid.ParamAttr(name="cls_seq_label_out_w",
                                                     initializer=fluid.initializer.TruncatedNormal(scale=0.02)),
                          bias_attr=fluid.ParamAttr(name="cls_seq_label_out_b",
                                                    initializer=fluid.initializer.Constant(0.)))

    ret_labels = fluid.layers.reshape(x=labels, shape=[-1, 1])
    ret_infers = fluid.layers.reshape(x=fluid.layers.argmax(log, axis=2), shape=[-1, 1])

    graph_vars = {"labels": ret_labels, "infers": ret_infers}
    for v in graph_vars.values():
        v.persistable = True

    return reader, graph_vars


def match(words, init_st: list):
    """抽取实体函数
    :param words:需要抽取的文字
    :param init_st:初始化参数。st()
    :return:数字列表，这些数字是在label_map_config中配置的
    """
    init_st[2].decorate_tensor_provider(data_generator(words))
    number = evaluate(*init_st)
    return number


def st(new_model_path=None, new_config=None, new_label_map_config=None) -> list:
    """初始化模型，只需要加载一次即可
    :param new_model_path: 模型路径
    :param new_config: 模型配置参数
    :param new_label_map_config: 模型实体映射
    """
    global model_path, config, label_map_config

    if new_model_path:
        model_path = new_model_path

    if new_config:
        config = new_config

    if new_label_map_config:
        label_map_config = new_label_map_config

    exe = fluid.Executor(fluid.CPUPlace())
    startup_program = fluid.Program()
    test_program = fluid.Program()
    with fluid.program_guard(test_program, startup_program):
        with fluid.unique_name.guard():
            test_reader, graph_vars = create_model()
    test_program = test_program.clone(for_test=True)
    exe.run(startup_program)
    init_checkpoint(exe, model_path, main_program=startup_program)
    return [exe, test_program, test_reader, graph_vars]


if __name__ == '__main__':
    # 默认的模型参数和映射表
    ERNIE_MODEL_PATH = 'D://model'
    s = st(ERNIE_MODEL_PATH)
    print(match('我叫刘万光我是贵阳市南明村永乐乡水塘村的村民', s))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 15:08
# @Author: Jtyoui@qq.com

from jtyoui.decorator import parameter_set_length
from collections import Counter

"""
肯德尔相关性系数
肯德尔相关系数是以Maurice Kendall命名的，并经常用希腊字母τ（tau）表示其值。
肯德尔相关系数是一个用来测量两个随机变量相关性的统计值。
一个肯德尔检验是一个无参数假设检验，它使用计算而得的相关系数去检验两个随机变量的统计依赖性。
肯德尔相关系数的取值范围在-1到1之间，当τ为1时，表示两个随机变量拥有一致的等级相关性；
当τ为-1时，表示两个随机变量拥有完全相反的等级相关性；当τ为0时，表示两个随机变量是相互独立的

举个例子。比如评委对选手的评分（优、中、差等），我们想看两个（或者多个）评委对几位选手的评价标准是否一致；
或者医院的尿糖化验报告，想检验各个医院对尿糖的化验结果是否一致，这时候就可以使用肯德尔相关性系数进行衡量。
"""


@parameter_set_length
def kendall_coefficient(sample_x, sample_y):
    """
    肯德尔相关性系数
    其中C表示XY中拥有一致性的元素对数（两个元素为一对）；D表示XY中拥有不一致性的元素对数。
    n1,n2,n3可以参考 https://blog.csdn.net/wsywl/article/details/5889419
    :param sample_x:数据集合
    :param sample_y:数据集合
    :return:两个集合的肯德尔相关性系数
    """
    if not (isinstance(sample_y, set) and isinstance(sample_x, set)):
        set_x, set_y = set(sample_x), set(sample_y)
    else:
        set_x, set_y = sample_x, sample_y
    n = len(sample_x)
    intersection = set_x.intersection(set_y)  # 交集
    c = len(intersection)  # 一致性的元素个数
    union = set_x.union(set_y)  # 并集
    d = len(union) - c  # 不一致性的元素个数
    n3 = n * (n - 1) / 2
    count_x, count_y = Counter(sample_x), Counter(sample_y)
    n1, n2 = 0, 0
    for i in intersection:
        n1 += count_x.get(i) * (count_x.get(i) - 1) / 2  # x中第i个小集合所包含的元素数
        n2 += count_y.get(i) * (count_y.get(i) - 1) / 2  # y中第i个小集合所包含的元素数
    return (c - d) * 2 / (pow((n3 - n1) * (n3 - n2), 0.5))


if __name__ == '__main__':
    print(kendall_coefficient([3, 1, 2, 2, 1, 3], [1, 2, 3, 2, 1, 1]))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/4/28 21:56
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from html.parser import HTMLParser
from urllib.parse import quote
import requests
from jtyoui.web import headers_ua
import re


class _InfoSearch(HTMLParser):
    """基类"""

    def __init__(self):
        HTMLParser.__init__(self)
        self._data_flag = ''
        self.desc = ''
        self._desc_flag = False
        self.info_name = []
        self.info_value = []
        self._info_name = False
        self._info_value = False
        self.info = {}

    def handle_starttag(self, tag, attrs):  # 开始标签
        if len(attrs) > 0:
            attr = attrs[0]
        else:
            return
        if tag == 'div' and 'lemma-summary' in attr:
            self._desc_flag = True
        elif tag == 'dt' and 'basicInfo-item name' in attr:
            self._info_name = True
        elif tag == 'dd' and 'basicInfo-item value' in attr:
            self._info_value = True
        elif tag == 'div' and 'configModuleBanner' in attr:
            self._desc_flag = False
        else:
            return
        self._data_flag = ''

    def handle_endtag(self, tag):  # 结束标签
        _data = self._data_flag.replace('\n', '').replace(u'\xa0', '')
        if tag == 'dt':
            self._info_name = False
            if _data:
                self.info_name.append(_data)
        elif tag == 'dd':
            self._info_value = False
            if _data:
                self.info_value.append(_data)

    def handle_data(self, data):  # 内容
        if self._desc_flag:
            self.desc += data
        elif self._info_name:
            self._data_flag += data
        elif self._info_value:
            self._data_flag += data

    def basic_info(self):  # 基本信息
        for k, v in zip(self.info_name, self.info_value):
            if k == v:
                continue
            self.info[k] = v
        return self.info

    def describe(self):  # 摘要
        return self.desc.replace('\n', '').replace(u'\xa0', '')


class BaiDuInfoSearch:
    """百度百科搜索基本信息"""

    def __init__(self, data):
        if '<html>' not in data:
            data = Load_BaiDuBaiKe(data)
        self._BD = _InfoSearch()
        self._BD.feed(data)

    def info(self):
        """基本信息"""
        info = self._BD.basic_info()
        info = re.sub(r'\[\d*-?\d*\]', '', str(info))
        return eval(info)

    def desc(self):
        """描述信息"""
        return re.sub(r'\[\d*-?\d*\]', '', self._BD.describe())


def Load_BaiDuBaiKe(name):
    """下载百度百科里面的内容信息
    :param name: 百科百科名字
    :return: 百度百科的文本信息
    """
    url = F'https://baike.baidu.com/item/{quote(name)}'
    response = requests.get(url, headers=headers_ua())
    data = response.content.decode('utf-8')
    return data


if __name__ == '__main__':
    import pprint

    bd = BaiDuInfoSearch('玛卡')
    print(bd.desc())
    pprint.pprint(bd.info())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/4 0:06
# @Author: jtyoui@qq.com
from collections import Counter
import pickle


def analysis_vocab(vocab_path, save_vocab_model_path, frequency=10):
    """根据已知文本统计词向量，取频率大于10的文字，词索引根据频率大小进行排序"""
    v = []
    with open(file=vocab_path) as fp:
        for line in fp:
            data = line.strip().split('_')
            v.extend(data)
    c = Counter(v)
    f = filter(lambda x: x[1] >= frequency, c.items())
    s = sorted(f, key=lambda x: x[1], reverse=True)
    # 填充是0、开始是1、结束是2、掩码是3、未知是UNK
    vocab = {'[PAD]': 0, '[UNK]': 1}
    for index, (k, _) in enumerate(iterable=s, start=len(vocab)):
        vocab[k] = index
    with open(file=save_vocab_model_path, mode='wb') as fp:
        pickle.dump(obj=vocab, file=fp)


def load_vocab(load_vocab_model_path):
    """加载模型"""
    with open(file=load_vocab_model_path, mode='rb') as fp:
        return pickle.load(file=fp)


def vocab_test(test_path, vocab, max_chunk_length):
    test, original = [], []
    with open(test_path)as fp:
        for line in fp:
            line = line.strip().split('_')
            original.append(line)
            m = list(map(lambda x: vocab.get(x, vocab['[UNK]']), line))
            if len(m) <= max_chunk_length:
                data = [vocab['[PAD]']] * (max_chunk_length - len(m)) + m
            else:
                data = m[:max_chunk_length]
            test.append(data)
    return test, original


def analysis_rational_len(train_or_dev_path, percent=0.9):
    """分析文本之间的长度分布，percent表示分布率"""
    length = []
    with open(file=train_or_dev_path)as fp:
        for lines in fp:
            lines = lines.replace('  ', '_')
            length.append(lines.count('_') + 1)
    rational_len = min(length)
    while True:
        f = filter(lambda x: rational_len - x >= 0, length)
        t = tuple(f)
        if len(t) / len(length) >= percent:
            return rational_len
        else:
            rational_len += 1


def vocab_train_label(train_or_dev_path, vocab, tags, max_chunk_length):
    """将训练数据转为格式化
    :param train_or_dev_path: 训练数据路径
    :param vocab: 词向量
    :param tags: 序列标签
    :param max_chunk_length: 最大序列词语长度
    :return: 训练格式和测试格式
    """
    trains, labels = [], []
    with open(file=train_or_dev_path)as fp:
        for lines in fp:
            data, sequence = [], []
            line = lines.strip().split('  ')
            ms = map(lambda x: (x[:-2].split('_'), x[-1]), line)
            for v, k in ms:
                data.extend(map(lambda x: vocab.get(x, vocab['[UNK]']), v))
                if k != 'o':
                    m = map(lambda x: tags[x], ['B-' + k] + [('I-' + k)] * (len(v) - 1))
                else:
                    m = [tags['O']] * len(v)
                sequence.extend(m)
            data_len = len(data)
            if data_len <= max_chunk_length:
                data = [vocab['[PAD]']] * (max_chunk_length - data_len) + data
                sequence = [-1] * (max_chunk_length - data_len) + sequence
            else:
                data = data[:max_chunk_length]
                sequence = sequence[:max_chunk_length]
            trains.append(data)
            labels.append(sequence)
    return trains, labels


def restore_format(crf_path, standard_path):
    f_write = open(standard_path, 'w', newline='\n')
    with open(crf_path, 'r', newline='\n') as fp:
        lines = fp.read().split('\n\n')
    for line in lines:
        if line == '':
            continue
        tokens = line.split('\n')
        features = []
        tags = []
        for token in tokens:
            feature_tag = token.split()
            features.append(feature_tag[0])
            tags.append(feature_tag[-1])
        samples = []
        i = 0
        while i < len(features):
            sample = []
            if tags[i] == 'O':
                sample.append(features[i])
                j = i + 1
                while j < len(features) and tags[j] == 'O':
                    sample.append(features[j])
                    j += 1
                samples.append('_'.join(sample) + '/o')
            else:
                if tags[i][0] != 'B':
                    print(tags[i][0] + ' error start')
                    j = i + 1
                else:
                    sample.append(features[i])
                    j = i + 1
                    while j < len(features) and tags[j][0] == 'I' and tags[j][-1] == tags[i][-1]:
                        sample.append(features[j])
                        j += 1
                    samples.append('_'.join(sample) + '/' + tags[i][-1])
            i = j
        f_write.write('  '.join(samples) + '\n')
    f_write.close()

# if __name__ == '__main__':
#     analysis_vocab('./data/corpus.txt', './vocab.pkl')
#     tag = {'O': 0, 'B-a': 1, 'I-a': 2, 'B-b': 3, 'I-b': 4, 'B-c': 5, 'I-c': 6}
#     vocabs = load_vocab('vocab.pkl')
#     leg = analysis_rational_len('./data/train.txt', 0.95)
#     train, label = vocab_train_label('./data/train.txt', vocab=vocabs, tags=tag, max_chunk_length=leg)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/23 16:20
# @Author: Jtyoui@qq.com
import os


def format_CRF_to_ERNIE(load_file, save_file, newline=os.linesep):
    """将CRF训练的格式转为ERNIE训练的格式
    :param load_file:加载数据的文件地址
    :param save_file:保存数据的文件地址
    :param newline:文件的换行符,默认是本机系统的换行符
    【CRF】
    中	B-ORG
    共	I-ORG
    中	I-ORG
    央	I-ORG
    致	O
    中	B-ORG
    国	I-ORG
    致	I-ORG
    公	I-ORG
    党	I-ORG
    十	I-ORG
    一	I-ORG
    大	I-ORG
    的	O
    贺	O
    词	O
    【ERNIE】
    中\x02共\x02中\x02央\x02致\x02中\x02国\x02致\x02公\x02党\x02十\x02一\x02大\x02的\x02贺\x02词    B-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02O\x02B-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02I-ORG\x02O\x02O\x02O
    :return: 文件地址
    """
    ws = open(save_file, encoding='utf-8', mode='w', newline='\n')
    ws.write('text_a\tlabel\n')
    with open(load_file, encoding='utf-8', newline=newline)as fp:
        data = fp.read().strip().split(newline + newline)
        for line in data:
            chars, labels = [], []
            ls = line.split(newline)
            for c in ls:
                char, label = c.split()
                chars.append(char)
                labels.append(label)
            ws.write('\x02'.join(chars) + '\t' + '\x02'.join(labels) + '\n')
    ws.close()


if __name__ == '__main__':
    format_CRF_to_ERNIE('train_data.txt', 'train.tsv')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/20 18:12
# @Author: Jtyoui@qq.com
from jtyoui import parameter_set_length

"""
杰卡德距离
杰卡德距离是用来衡量两个集合差异性的一种指标，它是杰卡德相似系数的补集，
被定义为1减去杰卡德相似系数。而杰卡德相似系数，
也称杰卡德指数,是用来衡量两个集合相似度的一种指标
"""


@parameter_set_length
def jaccard_distance(set_, other_set):
    """
    杰卡德距离
    1）  m00:代表向量A与向量B都是0的维度个数；
    2）  m01:代表向量A是0而向量B是1的维度个数；
    3）  m10:代表向量A是1而向量B是0的维度个数；
    4)   m11:代表向量A和向量B都是1的维度个数。
    n维向量的每一维都会落入这4类中的某一类，因此：
    Jaccard距离为:m01+m10/m01+m10+m11+m00
    :param set_:以1和0组合的集合
    :param other_set:以1和0组合的集合
    :return:两个不一样的频率
    """
    m00, m01, m10, m11 = 0, 0, 0, 0
    for x, y in zip(set_, other_set):
        if x == y == 0:
            m00 += 1
        elif x == 1 and y == 0:
            m10 += 1
        elif x == 0 and y == 1:
            m01 += 1
        else:
            m11 += 1
    return (m01 + m10) / (m01 + m10 + m11 + m00)


def jaccard_set_distance(set_, other_set):
    """
    杰卡德距离
    两个集合A和B的交集元素在A，B的并集中所占的比例，
    称为两个集合的杰卡德相似系数
    与杰卡德相似系数相反的概念是杰卡德距离
    杰卡德距离用两个集合中不同元素占所有元素的比例来衡量两个集合的区分度
    :param set_:一个set类型
    :param other_set:另一个set类型
    :return:集合之间的值
    """
    if not (isinstance(set_, set) and isinstance(other_set, set)):
        raise TypeError("参数必须是set类型")
    intersection = len(set_.intersection(other_set))
    union = len(set_.union(other_set))
    return (union - intersection) / union


if __name__ == '__main__':
    print(jaccard_distance([0, 1, 1, 1, 0, 0, 1, 1, 0, 1], [1, 1, 1, 1, 0, 0, 1, 1, 0, 0]))
    print(jaccard_set_distance({1, 10, 31, 2, 13, 26, 43}, {21, 52, 31, 43, 6, 12, 31}))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:37
# @Author: Jtyoui@qq.com

"""
编辑距离
编辑距离又称Levenshtein距离，是指两个字串之间，由一个转成另一个所需的最少编辑操作次数。
许可的编辑操作包括将一个字符替换成另一个字符，插入一个字符，删除一个字符。一般来说，
编辑距离越小，两个串的相似度越大。
"""


def edit_distance(chars, other_chars):
    """
    编辑距离
    :param chars: 字符串
    :param other_chars: 另一个字符串
    :return: 编辑距离值
    """
    w, h = len(chars), len(other_chars)  # 计算出长和宽
    array = [[0 for _ in range(h + 1)] for _ in range(w + 1)]  # 初始化
    for i in range(w + 1):
        for j in range(h + 1):
            array[i][j] = i + j if i == 0 or j == 0 else min(array[i - 1][j - 1], array[i - 1][j], array[i][j - 1]) + 1
            array[i][j] = array[i - 1][j - 1] if chars[i - 1] == other_chars[j - 1] else array[i][j]
    return array[w][h]


if __name__ == '__main__':
    print(edit_distance('我吃饭了', '我正在吃饭'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/19
# @Email : jtyoui@qq.com


# 常见的照片格式
Photo_Format = (
    'bmp', 'jpg', 'png', 'tif', 'gif', 'pcx', 'tga', 'exif', 'fpx', 'svg', 'psd', 'cdr', 'pcd', 'dxf', 'ufo', 'eps',
    'ai', 'raw', 'WMF', 'webp'
)

# 常见的文字编码格式
Decode = ('Unicode', 'ASCII', 'GBK', 'GB2312', 'UTF-8', 'ISO-8859-1', 'UTF-16', 'GB18030', 'ISO-8859-2')


# 数学符号
class MathSymbols:
    """‖‰℃℉←↑→↓∈∏∑°√∝∞∟∠∣∧∨∩∪∫∮～≈≌≒≠≡"""
    vector_value = '‖'  # ‖A‖ 表示A向量的值
    one_thousand = '‰'  # 千分号
    celsius_scale = '℃'  # 摄氏温标的温度计量单位
    fahrenheit_scale = '℉'  # 华氏温标
    right = '→'  # 向右
    left = '←'  # 向左
    up = '↑'  # 向上
    down = '↓'  # 向下
    belong_to = '∈'  # 属于
    product = '∏'  # 求乘积
    summation = '∑'  # 求累加
    one_degrees = '°'  # 1度,度角单位
    check_mark = '√'  # 对钩
    positive_proportion = '∝'
    infinity = '∞'
    slope = '∠'
    intersection = '∩'  # 交集
    and_ = '∧'  # 逻辑和
    condition = '∣'  # 条件概率P(A∣B)
    union = '∪'  # 并集
    integral = '∫'  # 积分
    or_ = '∨'  # 逻辑或
    closed_curve = '∮'  # 闭合曲线
    asymptotically_equal = '～'  # 逐渐相等,f(x)～g(x),表示lim f(x)=lim g(x)
    approximately_equal = '≈'  # 约等于
    identically_equal = '≌'  # 全等
    reversible = '≒'  # 可逆
    not_equal = '≠'  # 不等于
    identity = '≡'  # 恒等于
    ls = list('‖‰℃℉←↑→↓∈∏∑°√∝∞∟∠∣∧∨∩∪∫∮～≈≌≒≠≡')


# 将英文的星期转为中文
week_to_chinese = {
    'Monday': '星期一',
    'Mon': '星期一',
    'Mon.': '星期一',
    'Tuesday': '星期二',
    'Tues': '星期二',
    'Tues.': '星期二',
    'Tue': '星期二',
    'Wednesday': '星期三',
    'Wed': '星期三',
    'Wed.': '星期三',
    'Thursday': '星期四',
    'Thur': '星期四',
    'Thu': '星期四',
    'Thur.': '星期四',
    'Thurs': '星期四',
    'Thurs.': '星期四',
    'Friday': '星期五',
    'Fri': '星期五',
    'Fri.': '星期五',
    'Saturday': '星期六',
    'Sat': '星期六',
    'Sat.': '星期六',
    'Sunday': '星期日',
    'Sun': '星期日',
    'Sun.': '星期日',
}

# 将中文的星期转为英文
week_to_english = {
    '星期一': 'Monday',
    '星期二': 'Tuesday',
    '星期三': 'Wednesday',
    '星期四': 'Thursday',
    '星期五': 'Friday',
    '星期六': 'Saturday',
    '星期日': 'Sunday',
}

# 将英文的月份转为中文
month_to_chinese = {
    'January': '一月',
    'Jan': '一月',
    'Jan.': '一月',
    'February': '二月',
    'Feb': '二月',
    'Feb.': '二月',
    'March': '三月',
    'Mar': '三月',
    'Mar.': '三月',
    'April': '四月',
    'Apr': '四月',
    'Apr.': '四月',
    'May': '五月',
    'May.': '五月',
    'June': '六月',
    'Jun': '六月',
    'Jun.': '六月',
    'July': '七月',
    'Jul': '七月',
    'Jul.': '七月',
    'August': '八月',
    'Aug': '八月',
    'Aug.': '八月',
    'September': '九月',
    'Sept': '九月',
    'Sept.': '九月',
    'October': '十月',
    'Oct': '十月',
    'Oct.': '十月',
    'November': '十一月',
    'Nov': '十一月',
    'Nov.': '十一月',
    'December': '十二月',
    'Dec': '十二月',
    'Dec.': '十二月'
}

# 将中文的月份转为英文
month_to_english = {
    '一月': 'January',
    '二月': 'February',
    '三月': 'March',
    '四月': 'April',
    '五月': 'May',
    '六月': 'June',
    '七月': 'July',
    '八月': 'August',
    '九月': 'September',
    '十月': 'October',
    '十一月': 'November',
    '十二月': 'December'
}

# 翻译http状态码的含义
http_status_code = {
    '100': "请求者应当继续提出请求。服务器返回此代码表示已收到请求的第一部分，正在等待其余部分。",
    '101': "请求者已要求服务器切换协议，服务器已确认并准备切换。",
    '200': '请求成功。',
    '201': '已创建，成功请求并创建了新的资源。',
    '202': '已接受，已经接受请求，但未处理完成。',
    '203': '非授权信息，请求成功。',
    '204': '无内容，服务器成功处理，但未返回内容。在未更新网页的情况下，可确保浏览器继续显示当前文档。',
    '205': '重置内容，服务器处理成功，用户终端（例如：浏览器）应重置文档视图。可通过此返回码清除浏览器的表单域。',
    '206': '部分内容，服务器成功处理了部分GET请求。',
    '300': "针对请求，服务器可执行多种操作。服务器可根据请求者选择一项操作，或提供操作列表供请求者选择。",
    '301': "请求的网页已永久移动到新位置，服务器返回此响应（对GET或HEAD请求的响应）时，会自动将请求者转到新位置。",
    '302': "服务器目前从不同位置的网页响应请求，但请求者应继续使用原有位置来进行以后的请求。",
    '303': "请求者应当对不同的位置使用单独的GET请求来检索响应时，服务器返回此代码。",
    '304': "自从上次请求后，请求的网页未修改过。服务器返回此响应时，不会返回网页内容。",
    '305': "请求者只能使用代理访问请求的网页。如果服务器返回此响应，还表示请求者应使用代理。",
    '306': '已经被废弃的HTTP状态码。',
    '307': "服务器目前从不同位置的网页响应请求，但请求者应继续使用原有位置来进行以后的请求。",
    '400': "服务器不理解请求的语法。",
    '401': "请求要求身份验证。对于需要登录的网页，服务器可能返回此响应。",
    '402': '保留，将来使用。',
    '403': "服务器拒绝请求。",
    '404': "服务器找不到请求的网页。",
    '405': "禁用请求中指定的方法。",
    '406': "无法使用请求的内容特性响应请求的网页。",
    '407': "请求者应当授权使用代理。",
    '408': "服务器等候请求时发生超时。",
    '409': "服务器在完成请求时发生冲突。服务器必须在响应中包含有关冲突的信息。",
    '410': "如果请求的资源已永久删除，服务器就会返回此响应。",
    '411': "服务器不接受不含有效内容长度标头字段的请求。",
    '412': "服务器未满足请求者在请求中设置的其中一个前提条件。",
    '413': "服务器无法处理请求，因为请求实体过大，超出服务器的处理能力。",
    '414': "请求的URI过长，服务器无法处理。",
    '415': "请求的格式不受请求页面的支持。",
    '416': "如果页面无法提供请求的范围，则服务器会返回此状态代码。",
    '417': "服务器未满足期望':请求标头字段的要求。",
    '500': "服务器遇到错误，无法完成请求。",
    '501': "服务器不具备完成请求的功能。例如，服务器无法识别请求方法时可能会返回此代码。",
    '502': "服务器作为网关或代理，从上游服务器收到无效响应。",
    '503': "服务器目前无法使用（由于超载或停机维护）。通常这只是暂时状态。",
    '504': "服务器作为网关或代理，但是没有及时从上游服务器收到请求。",
    '505': "服务器不支持请求中所用的HTTP协议版本。",
}

# 将拼音声调去掉
letter_maps = {'ā': 'a',
               'á': 'a',
               'a': 'a',
               'ǎ': 'a',
               'à': 'a',
               'e': 'e',
               'ē': 'e',
               'é': 'e',
               'ě': 'e',
               'è': 'e',
               'o': 'o',
               'ō': 'o',
               'ó': 'o',
               'ǒ': 'o',
               'ò': 'o',
               'm': 'm',
               'ǹ': 'n',
               'ň': 'n',
               'ǚ': 'u',
               'g': 'g',
               'ī': 'i',
               'k': 'k',
               'ń': 'n',
               'ǘ': 'u',
               'y': 'y',
               'ǔ': 'u',
               'ū': 'u',
               'ǜ': 'u',
               'x': 'x',
               'n': 'n',
               'd': 'd',
               'ú': 'u',
               'r': 'r',
               'c': 'c',
               'j': 'j',
               'i': 'i',
               'ü': 'u',
               'w': 'w',
               's': 's',
               'b': 'b',
               'l': 'l',
               'h': 'h',
               'ǐ': 'i',
               'í': 'i',
               'ḿ': 'm',
               'p': 'p',
               'f': 'f',
               'u': 'u',
               'q': 'q',
               'z': 'z',
               'ì': 'i',
               'ù': 'u',
               't': 't'
               }

# 节日
holiday = {
    "01-01": "元旦节",
    "腊月卅十": "除夕",
    "正月除一": "春节",
    "正月十五": "元宵节",
    "02-14": "情人节",
    "03-08": "妇女节",
    "03-12": "植树节",
    "04-01": "愚人节",
    "04-05": "清明节",
    "五月除五": "端午节",
    "05-01": "劳动节",
    "05-04": "青年节",
    "06-01": "儿童节",
    "07-01": "建党节",
    "七月除七": "七夕节",
    "08-01": "建军节",
    "七月十五": "中元节",
    "八月十五": "中秋节",
    "九月除九": "重阳节",
    "09-10": "教师节",
    "10-01": "国庆节",
    "11-01": "万圣节",
    "12-24": "平安夜",
    "12-25": "圣诞节"
}

# 中国的月份
chinese_mon = ["零", "正", "二", "三", "四", "五", "六", "七", "八", "九", "十", "冬", "腊"]

# 中国的月份对应数字
chinese_mon_number = {
    '零': '0',
    '正': '1',
    '一': '1',
    '二': '2',
    '两': '2',
    '三': '3',
    '四': '4',
    '五': '5',
    '六': '6',
    '七': '7',
    '八': '8',
    '九': '9',
    '十': '10',
    '冬': '11',
    '腊': '12',
}

# 增加时间
add_time = {
    '天': 1,
    '前天': -2,
    '昨天': -1,
    '今天': 0,
    '明天': 1,
    '后天': 2,
    '去年': -1 * 365,
    '前年': -2 * 365,
    '昨年': -1 * 365,
    '今年': 0,
    '明年': 1 * 365,
    '后年': 2 * 365,
    '上个月': -1 * 31,
    '这个月': 0,
    '下个月': 1 * 31,
    '上月': -1 * 31,
    '这月': 0,
    '下月': 1 * 31,
    '下周': 7,
    '上周': -7,
    '下个周': 7,
    '上个周': -7,
    '这周': 0,
    '这个周': 0,
}

# 中文数字
num_symbol = ['一', '二', '两', '三', '四', '五', '六', '七', '八', '九', '十']
more_num_symbol = ['零', '百', '千', '万', '亿', '兆'] + num_symbol

# 模糊音
fuzzy_tone = {
    'z': 'zh',
    'l': 'n',
    'c': 'ch',
    'f': 'h',
    's': 'sh',
    'r': 'l',
    'en': 'eng',
    'in': 'ing',
    'an': 'ang',
    'ei': 'ui'
}

# 中文空格
chinese_blank_space = '\u3000'
chinese_blank_spaces = '　'

if __name__ == '__main__':
    print(MathSymbols.intersection)
    print(MathSymbols.ls)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/4 1:50
# @Author: jtyoui@qq.com
from keras.models import Sequential, load_model
from keras.layers import Embedding, Bidirectional, LSTM, SpatialDropout1D, TimeDistributed, Dense
from keras_contrib.layers import CRF, crf
from keras.optimizers import Adam
from keras.utils.np_utils import np
from keras.callbacks import RemoteMonitor
from jtyoui.neuralNetwork.kerase.AnalyticalData import *
from keras.activations import relu

tag = {'O': 0, 'B-a': 1, 'I-a': 2, 'B-b': 3, 'I-b': 4, 'B-c': 5, 'I-c': 6}
dirs = r'C:\Users\Xiaoi\Desktop\datagrand\datagrand\\'
vocab_path_model = dirs + 'data/vocab.pkl'
train_vocab_path = dirs + 'data/corpus.txt'
train_path = dirs + 'data/train.txt'
test_path = dirs + 'data/test.txt'
temp_path = dirs + 'data/crf.txt'
result_path = dirs + 'data/result.txt'
model_path = dirs + 'data/model-ner.h5'
cxl_model = dirs + r'data/cxl.pkl'

# analysis_vocab(train_vocab_path, vocab_path_model)  # 训练vocab，只需要训练一次就够了
vocab = load_vocab(vocab_path_model)
length = analysis_rational_len(train_path, percent=0.93)


def load_embedding():
    with open(cxl_model, 'rb') as fp:
        cxl = pickle.load(fp)
    embedding_matrix = np.zeros((len(vocab), 300))
    for k, v in vocab.items():
        if cxl.get(k) is not None:
            embedding_matrix[v] = cxl[k]
    return embedding_matrix


def train_model():
    if cxl_model:
        embedding_matrix = load_embedding()
    else:
        embedding_matrix = {}
    train, label = vocab_train_label(train_path, vocab=vocab, tags=tag, max_chunk_length=length)
    n = np.array(label, dtype=np.float)
    labels = n.reshape((n.shape[0], n.shape[1], 1))
    model = Sequential([
        Embedding(input_dim=len(vocab), output_dim=300, mask_zero=True, input_length=length, weights=[embedding_matrix],
                  trainable=False),
        SpatialDropout1D(0.2),
        Bidirectional(layer=LSTM(units=150, return_sequences=True, dropout=0.2, recurrent_dropout=0.2)),
        TimeDistributed(Dense(len(tag), activation=relu)),
    ])
    crf_ = CRF(units=len(tag), sparse_target=True)
    model.add(crf_)
    model.compile(optimizer=Adam(), loss=crf_.loss_function, metrics=[crf_.accuracy])
    model.fit(x=np.array(train), y=labels, batch_size=16, epochs=4, callbacks=[RemoteMonitor()])
    model.save(model_path)


def predict_model():
    x_test, original = vocab_test(test_path, vocab, length)
    ws = open(temp_path, mode='w', newline='\n')
    tags = dict(zip(tag.values(), tag.keys()))
    custom_objects = {'CRF': CRF, 'crf_loss': crf.crf_loss, 'crf_viterbi_accuracy': crf.crf_viterbi_accuracy}
    model = load_model(model_path, custom_objects=custom_objects)
    for question, tests in zip(original, x_test):
        raw = model.predict([[tests]])[0][-len(question):]
        result = [np.argmax(row) for row in raw]
        answer = tuple(map(lambda x: tags[x], result))
        ma = map(lambda x: x[0] + '\t' + x[1] + '\n', zip(question, answer))
        ws.writelines(ma)
        ws.write('\n')
    ws.flush()
    ws.close()


if __name__ == '__main__':
    train_model()
    predict_model()
    restore_format(crf_path=temp_path, standard_path=result_path)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/28 1:14
# @Author: jtyoui@qq.com
from .ernie import st as ernie_st, match as ernie_match, ERNIE_MODEL_PARAMETER, ERNIE_LABEL_MAP  # 实体识别



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/14 16:37
# @Author: Jtyoui@qq.com
import tkinter


class Calculator(tkinter.Frame):
    def __init__(self):
        self.master = tkinter.Tk()
        self.master.title('计算器')
        self.master.resizable(width=False, height=False)  # 防止大小调整
        self.master.geometry('200x240+200+300')
        super().__init__(master=self.master)
        self.pack()
        self._create_widget()
        self.master.mainloop()

    def _create_widget(self):
        """通过grid实现计算器界面"""
        btn = (['MC', 'M+', 'M-', 'MR'],
               ['C', '±', '/', 'X'],
               [7, 8, 9, '-'],
               [4, 5, 6, '+'],
               [1, 2, 3, '='],
               [0, '.'])
        tkinter.Entry(self).grid(row=0, column=0, columnspan=4, pady=10)
        for row_index, row in enumerate(btn, 1):
            for c_index, c in enumerate(row):
                self.b = tkinter.Button(self, text=c, width=3, height=1, command=self.logic)
                if c == '=':
                    self.b.grid(row=row_index, column=c_index, rowspan=2, sticky=tkinter.NSEW)
                elif c == 0:
                    self.b.grid(row=row_index, column=c_index, columnspan=2, sticky=tkinter.NSEW)
                elif c == '.':
                    self.b.grid(row=row_index, column=c_index + 1, sticky=tkinter.NSEW)
                else:
                    self.b.grid(row=row_index, column=c_index, sticky=tkinter.NSEW)

    def logic(self):
        print(self.b)


if __name__ == '__main__':
    Calculator()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/11
"""
科赫雪花
在一单位长度的线段上对其三等分，将中间段直线换成一个去掉底边的等边三角形，
再在每条直线上重复以上操作，如此进行下去直到无穷，就得到分形Koch曲线。
"""

import turtle


def koch(p, order, size):
    if order == 0:
        p.forward(size)
    else:
        for angle in [60, -120, 60, 0]:
            koch(p, order - 1, size / 3)
            p.left(angle)


def draw_koch(n=3, polygon=6):
    """
    绘画科赫雪花
    :param n: 迭代次数
    :param polygon: 多边形雪花
    """
    pen = turtle.Pen()
    turtle.title('科赫雪花')
    pen.speed(0)
    r = pen.screen.canvwidth / 2
    pen.penup()
    pen.goto(-r, 0)
    pen.pendown()
    a = 180 - (polygon - 2) * 180 / polygon  # 计算多边形旋转的角度,计算公式
    # 多边形的内角和公式（n-2）*180/n
    for i in range(polygon):
        angle = a * (1 - i)
        pen.setheading(angle)
        koch(pen, n, r)
    turtle.done()


if __name__ == '__main__':
    draw_koch(3, 7)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/10 9:15
# @Author: Jtyoui@qq.com
import os

"""
该方法属于大量日期转化。消耗内存为代价提高速度。该类只能查询1901-2099年。
"""
_CTC = {}
_TIANGAN_DIZHI = {}
_SC = {}


def download_date(download_address='date'):
    global _CTC, _TIANGAN_DIZHI, _SC
    if not os.path.exists(download_address + os.sep + 'date.zip'):
        from jtyoui.data import download_gitee
        b = download_gitee('packet', 'date.zip', download_address)
    else:
        b = 'success'
    if b == 'success':
        from jtyoui.compress import unzip
        ls = unzip(download_address + os.sep + 'date.zip', 'date.txt', sep='\r\n')
        for line in ls[:-1]:
            ctc, cs, td = line.split('\t')
            _CTC[ctc] = cs
            _SC[cs] = ctc
            if _TIANGAN_DIZHI.get(td):
                _TIANGAN_DIZHI[td].append(ctc)
            else:
                _TIANGAN_DIZHI[td] = [ctc]
    else:
        from jtyoui.error import DownloadDataExceptionError
        raise DownloadDataExceptionError('下载失败！请检查网络。')


def td_to_ctc(td):
    """天干地支纪年转农历
    :param td:输入一个天干地支纪年
    :return :农历
    """
    return _TIANGAN_DIZHI.get(td)


def td_to_sc(td):
    """天干地支纪年转阳历
    :param td:输入一个天干地支纪年
    :return :阳历
    """
    ctc = td_to_ctc(td)
    return [ctc_to_sc(c) for c in ctc]


def ctc_to_sc(ctc):
    """农历转阳历
    :param ctc: 农历
    :return: 阳历
    """
    return _CTC.get(ctc)


def ctc_to_td(ctc):
    """
    农历转天干地支
    :param ctc: 农历
    :return: 天干地支,找不到返回空
    """
    for td_, ctc_ in _TIANGAN_DIZHI.items():
        if ctc in ctc_:
            return td_
    return ''


def sc_to_ctc(sc):
    """阳历转农历
    :param sc: 阳历
    :return: 农历
    """
    return _SC.get(sc)


def sc_to_td(sc):
    """阳历转天干地支纪年
    :param sc: 阳历
    :return: 天干地支纪年
    """
    ctc = sc_to_ctc(sc)
    return ctc_to_td(ctc)


if __name__ == '__main__':
    download_date('temp')
    print('-----------------------------')
    # 农历
    print(ctc_to_sc('1984年闰十月初三'))  # 农历转阳历 1984年11月25日
    print(ctc_to_td('1984年闰十月初三'))  # 农历转天干地支 甲子年乙亥月癸亥日
    print('-----------------------------')
    # 阳历
    print(sc_to_ctc('1984年11月25日'))  # 阳历转农历 1984年闰十月初三
    print(sc_to_td('1984年11月25日'))  # 阳历转天干地支 甲子年乙亥月癸亥日
    print('-----------------------------')
    # 天干地支
    print(td_to_ctc('甲子年乙亥月癸亥日'))  # 天干地支转农历:['1984年闰十月初三', '2044年九月廿一']
    print(td_to_sc('甲子年乙亥月癸亥日'))  # 天干地支转阳历:['1984年11月25日', '2044年11月10日']



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2018/2/28 10:58
# @Author: Jtyoui@qq.com
import math

"""
NAA(New Word Analysis Algorithms):分析新词算法
"""


class NAA:
    all_lens = 0.0
    naa_words = dict()

    def __init__(self, split_num=4, seq=0.001, cond=50, free=0.5):
        """
        初始化参数
        :param split_num: 匹配个数
        :param seq: 关键字出现的频率
        :param cond: 凝固程度
        :param free: 自由程度
        """
        self.split_num = split_num
        self.seq = seq
        self.cond = cond
        self.free = free

    def read_ls(self, ls):
        st = ''.join(ls)
        self.all_lens = len(st)
        for word in ls:
            self.split(words=word)
        print("split over!")

    def __read_ls(self, ls):
        """
        读取链表，格式是[str,str,str,.....]一维链表
        :param ls: 链表
        """
        for word in ls:
            self.split(words=word)
        print("split over!")

    def read_string(self, st, split_seq='\n'):
        """
        讲字符按照split_seq格式来分割
        :param st: 字符串
        :param split_seq: 字符分割
        :return: None
        """
        self.all_lens = len(st)
        ls = st.split(split_seq)
        print("reader over!")
        self.__read_ls(ls=ls)

    def read_file(self, file, file_encoding='utf-8-sig'):
        """
        读取文件内容，注意文件是UTF-8的格式且不是BOM格式
        :param file: 读取的文件
        :param file_encoding: 文本编码
        """
        fp = open(file, "r", encoding=file_encoding)
        d = fp.read()
        self.read_string(st=d)
        fp.close()

    def split(self, words):
        """
        拆分字符，最大匹配num个字符，并也字典的形式返回，
        [出现次数,出现频率,凝固程度,自由程度,关键字的左邻,关键字的右邻](作为信息熵的衡量)
        """
        lens = len(words)
        for i in range(0, lens):
            for j in range(1, self.split_num + 1):
                if i + j < lens:
                    k = words[i:i + j]
                    word = self.naa_words.get(k)
                    if word:
                        word[0] += 1
                        word[1] = word[0] / self.all_lens
                        word[4].append(words[i - 1])
                        word[5].append(words[i + 1])
                    else:
                        self.naa_words[k] = [1, 1 / self.all_lens, 1, 0, [words[i - 1]], [words[i + 1]]]

    def handle(self):
        """
        处理数据
        计算左邻字集合和右邻字集合有多随机，左邻字信息熵和右邻字信息熵中的较小值
        计算凝固程度,自由程度
        """
        for k in self.naa_words:
            if len(k) == 1:
                continue
            word = self.naa_words[k]
            end_all = front_all = 0.0
            left = word[1] / (self.naa_words[k[0]][1] * self.naa_words[k[1:]][1])
            right = word[1] / (self.naa_words[k[-1]][1] * self.naa_words[k[:-1]][1])

            for front in word[4]:
                if front in self.naa_words:
                    front_all -= math.log(self.naa_words[front][1]) * self.naa_words[front][1]

            for end in word[5]:
                if end in self.naa_words:
                    end_all -= math.log(self.naa_words[end][1]) * self.naa_words[end][1]

            word[2] = left if left < right else right
            word[3] = front_all if front_all < end_all else end_all
        print("handle over!")

    def filter(self, filter_cond=10, filter_free=0.5, flag=False):
        """
        过滤一些不重要的数据
        [出现次数,出现频率,凝固程度,自由程度]
        :param filter_cond:  过滤凝聚度
        :param filter_free:  过滤自由度
        :param flag 是否是并且还是或者,默认是或者，满足一个就过滤
        :return: 过滤后的数据字典
        """
        key_words = dict()
        for k in self.naa_words:
            if len(k) <= 1:
                continue
            splits = self.naa_words[k]
            if splits[1] > self.seq and splits[2] > self.cond and splits[3] > self.free:
                key_words[k] = [splits[0], splits[1], splits[2], splits[3]]

        new_key = key_words.copy()
        keys = key_words.keys()
        for max_key in keys:
            """
                去除关键字像：中国新时，这种不能分成更多的词语
            """
            lens = len(max_key)
            if lens > 4:
                i = 2
                split_flag = 0
                while i <= lens:
                    if max_key[split_flag:i] in keys and max_key[split_flag:i] != max_key:
                        split_flag = i
                    i += 1
                if split_flag != lens:
                    new_key.pop(max_key)
                    continue
            for min_key in keys:
                """
                    去除子集
                """
                max_keys = key_words[max_key]
                if max_key in min_key and max_key != min_key:
                    if max_keys[0] <= key_words[min_key][0] + 5:
                        new_key.pop(max_key)
                        break
                    else:
                        if flag:
                            if max_keys[2] < filter_cond and max_keys[3] < filter_free:
                                new_key.pop(max_key)
                                break
                        elif max_keys[2] < filter_cond or max_keys[3] < filter_free:
                            new_key.pop(max_key)
                            break

        print("filter over!")
        return new_key


if __name__ == '__main__':
    naa = NAA(split_num=10, seq=0.0001)
    naa.read_file(file=r"D:\data.txt")
    naa.handle()
    data = naa.filter(filter_free=0.7, filter_cond=84, flag=False)
    for key, value in data.items():
        print(F'关键字:{key},次数:{value[0]},频率:{value[1]},凝聚度:{value[2]},自由度:{(value[3])}')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/16 17:45
# @Author: Jtyoui@qq.com
from types import FunctionType, ModuleType


def string_function(module: [str, ModuleType], func: str, *args, **kwargs):
    """根据字符串方法名来进行调用模块方法
    :param module: 模块名
    :param func: 模块中的方法名字
    :param args:方法里面的参数值
    :param kwargs:方法里面的参数值
    :return: 返回一个返回值
    """
    if isinstance(module, ModuleType):
        if hasattr(module, func):
            func_or_var = getattr(module, func)
            if isinstance(func_or_var, FunctionType):
                return func_or_var(*args, **kwargs)
            else:
                return func_or_var
        else:
            print(f'{func}不是一个方法')
    elif isinstance(module, str):
        m = __import__(module)
        return string_function(m, func, *args, **kwargs)
    else:
        print(f'{module}不是一个模块')
    return None


if __name__ == '__main__':
    print(string_function('jtyoui', 'BaiDuInfoSearch')('万绮雯').info())
    print(list(string_function('jtyoui', 'f_to_j', str_='載')))
    print(list(string_function('jtyoui', 'f_to_j', '載')))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/11
# @Email : jtyoui@qq.com
# @Software : PyCharm
from .CantorTernarySet import CantorTernarySet  # 康托三分集
from .KochSnowflake import draw_koch  # 科赫雪花



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 10:18
# @Author: Jtyoui@qq.com
from jtyoui import parameter_set_length

"""
布雷柯蒂斯距离
Bray Curtis距离主要用于生态学和环境科学，计算坐标之间的距离。
该距离取值在[0,1]之间。它也可以用来计算样本之间的差异。
"""


@parameter_set_length
def bray_curtis_distance(dimension_x, dimension_y):
    """
    布雷柯蒂斯距离
    :param dimension_x: 一个维度的集合,不是一个点
    :param dimension_y:另一个维度的集合.不是另一个点
    :return:布雷柯蒂斯距离
    """
    all_ = []
    for x, y in zip(dimension_x, dimension_y):
        all_.append(abs(x - y))
    return sum(all_) / (sum(dimension_y) + sum(dimension_x))


if __name__ == '__main__':
    print(bray_curtis_distance([11, 0, 7, 8, 0], [24, 37, 5, 18, 1]))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/8 10:10
# @Author: Jtyoui@qq.com
import re

# 普通车牌
ORDINARY_CAR_RE = "[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z][A-Z][A-HJ-NP-Z0-9]{4}[A-HJ-NP-Z0-9挂学警港澳]"
# 新能源车牌
NEW_ENERGY_VEHICLE_RE = "[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z][A-Z](?:(?:[0-9]{5}[DF])|(?:[DF][A-HJ-NP-Z0-9][0-9]{4}))"
# 所有的车牌
ALL_CAR_RE = ORDINARY_CAR_RE + "|" + NEW_ENERGY_VEHICLE_RE


def plate_number(number: str, car_re: str = ALL_CAR_RE) -> list:
    """车牌号验证
    :param car_re:匹配车牌号的正则，默认是：CAR_RE
    :param number:车牌号
    :return: 车牌号
    """
    return re.findall(car_re, number)


if __name__ == '__main__':
    print(plate_number('我家车牌号是：渝D134567'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/4/28 21:56
# @Email  : jtyoui@qq.com
# @Software: PyCharm

from .Encyclopedias import BaiDuInfoSearch, Load_BaiDuBaiKe  # 百度百科搜索基本信息
from .baiduwenku import BaiDuWenKu  # 增加百度文库下载



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/31 15:26
# @Author: Jtyoui@qq.com
from keras.datasets import mnist
from keras.utils import np_utils
from keras.models import Sequential, load_model
from keras.layers import Dense, Convolution2D, MaxPool2D, Flatten, Dropout
from keras.optimizers import SGD, Adam, RMSprop
from keras.layers.recurrent import SimpleRNN
from keras.losses import categorical_crossentropy
from keras.activations import tanh, softmax, relu
from keras.callbacks import RemoteMonitor
from keras.initializers import one, constant


def nn_model():
    (x_train, y_train), _ = mnist.load_data()
    # 归一化
    x_train = x_train.reshape(x_train.shape[0], -1) / 255.
    # one-hot
    y_train = np_utils.to_categorical(y=y_train, num_classes=10)
    # constant(value=1.)自定义常数，constant(value=1.)===one()
    # 创建模型:输入784个神经元，输出10个神经元
    model = Sequential([
        Dense(units=200, input_dim=784, bias_initializer=constant(value=1.), activation=tanh),
        Dense(units=100, bias_initializer=one(), activation=tanh),
        Dense(units=10, bias_initializer=one(), activation=softmax),
    ])

    opt = SGD(lr=0.2, clipnorm=1.)  # 优化器
    model.compile(optimizer=opt, loss=categorical_crossentropy, metrics=['acc', 'mae'])  # 编译
    model.fit(x_train, y_train, batch_size=64, epochs=20, callbacks=[RemoteMonitor()])
    model_save(model, './model.h5')


def cnn_model():
    (x_train, y_train), _ = mnist.load_data()
    # 归一化
    x_train = x_train.reshape(-1, 28, 28, 1) / 255.
    # one-hot
    y_train = np_utils.to_categorical(y=y_train, num_classes=10)

    model = Sequential([
        # input_shape:输入平面，就在第一个位置设置
        # filters：卷积核、滤波器
        # kernel_size：卷积核大小
        # strides：步长
        # padding有两种方式：same/valid
        # activation：激活函数
        Convolution2D(input_shape=(28, 28, 1), filters=32, kernel_size=5, strides=1, padding='same', activation=relu),
        MaxPool2D(pool_size=2, strides=2, padding='same'),
        Convolution2D(filters=64, kernel_size=5, padding='same', activation=relu),
        MaxPool2D(pool_size=2, trainable=2, padding='same'),
        Flatten(),  # 扁平化
        Dense(units=1024, activation=relu),
        Dropout(0.5),
        Dense(units=10, activation=softmax),
    ])
    opt = Adam(lr=1e-4)
    model.compile(optimizer=opt, loss=categorical_crossentropy, metrics=['accuracy'])
    model.fit(x=x_train, y=y_train, batch_size=64, epochs=20, callbacks=[RemoteMonitor()])
    model_save(model, './model.h5')


def rnn_model():
    (x_train, y_train), _ = mnist.load_data()
    # 归一化
    x_train = x_train / 255.
    # one-hot
    y_train = np_utils.to_categorical(y=y_train, num_classes=10)

    model = Sequential([
        SimpleRNN(units=50, input_shape=(28, 28)),
        Dense(units=10, activation=softmax),
    ])
    opt = RMSprop(lr=1e-4)
    model.compile(optimizer=opt, loss=categorical_crossentropy, metrics=['accuracy'])
    model.fit(x=x_train, y=y_train, batch_size=64, epochs=20, callbacks=[RemoteMonitor()])
    model_save(model, './model.h5')


def model_save(model, path):
    model.save(path)


def model_load(path='./model.h5'):
    model = load_model(path)
    return model


if __name__ == '__main__':
    nn_model()  # 神经网络
    # cnn_model()  # 卷积神经网络
    # rnn_model()  # 循环神经网络

    # # 加载模型
    # _, (x_test, y_test) = mnist.load_data()
    # x_test = x_test.reshape(x_test.shape[0], -1) / 255.
    # y_test = np_utils.to_categorical(y=y_test, num_classes=10)
    # data = x_test[0:10]  # 预测10张
    # label = y_test[0:10]
    # models = model_load()
    # pre = models.predict(x=data, batch_size=10)
    # print('实际：', np_utils.np.argmax(label, axis=1), '\t预测：', np_utils.np.argmax(pre, axis=1))

    # 创建模型图画
    # from keras.utils.vis_utils import plot_model
    # plot_model(model=model, to_file='rnn_model.png', show_shapes=True, rankdir='TB', show_layer_names='False')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 15:07
# @Author: Jtyoui@qq.com

from jtyoui.decorator import parameter_set_length

"""
斯皮尔曼相关系数
在统计学中, 以查尔斯·斯皮尔曼命名的斯皮尔曼等级相关系数，即SpearMan相关系数。
经常用希腊字母ρ表示。 它是衡量两个变量的依赖性的 非参数 指标。
它利用单调方程评价两个统计变量的相关性。 如果数据中没有重复值，
并且当两个变量完全单调相关时，斯皮尔曼相关系数则为+1或−1。
"""


@parameter_set_length
def spear_man_coefficient(sample_x, sample_y):
    """
    斯皮尔曼相关系数
    :param sample_x: x样本数据变量集合
    :param sample_y: y样本数据变量集合
    :return:x和y的相关系数
    """
    n = len(sample_x)
    sort_x = sorted(sample_x)  # 排序
    sort_y = sorted(sample_y)
    dict_x = dict(zip(sort_x, range(n)))  # 排序的字典
    dict_y = dict(zip(sort_y, range(n)))
    rank_x = [dict_x.get(x) for x in sample_x]  # x的秩
    rank_y = [dict_y.get(y) for y in sample_y]
    rank = [(x - y) ** 2 for x, y in zip(rank_x, rank_y)]  # 每一个秩的平方
    return 1 - 6 * sum(rank) / (n * (pow(n, 2) - 1))


if __name__ == '__main__':
    print(spear_man_coefficient((11, 490, 14, 43, 30, 3), (2, 75, 3, 44, 7, 42)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/5/9 11:47
# @Author: Jtyoui@qq.com

"""
CTC (Chinese traditional calendar)：农历
农历转阳历
"""
from jtyoui.plunar import SC


class CTC:

    def __init__(self, ctc_year, ctc_mon, ctc_day):
        """农历的年、农历的月、农历的日，如果是闰月在月的前面加负号， 比如：闰6月 -6
        :param ctc_year: 农历的年：2017（2017年）
        :param ctc_mon: 农历的月：-6 （闰6月）。6 （6月）
        :param ctc_day: 农历的日：8（初八）
        """
        self.ctc_year = ctc_year
        self.ctc_mon = abs(ctc_mon)
        self.ctc_day = ctc_day
        if ctc_mon < 0:
            self.raw_date = F'{ctc_year}年闰{self.ctc_mon}月{ctc_day}日'
        else:
            self.raw_date = F'{ctc_year}年{self.ctc_mon}月{ctc_day}日'
        self.sc_start_year = ctc_year - 1 if ctc_mon <= 3 else ctc_year
        self.sc_end_year = ctc_year + 1 if ctc_mon >= 10 else ctc_year
        self.sc_start_mon = (ctc_mon + 12 - 3) % 12
        self.sc_end_mon = (ctc_mon + 12 + 3) % 12
        self.sc_start_mon = self.sc_start_mon if self.sc_start_mon else 12
        self.sc_end_mon = self.sc_end_mon if self.sc_end_mon else 12
        self.date = None

    def find_sc(self):
        """返回阳历,如果找不倒返回None"""
        date = []
        if self.sc_start_year == self.sc_end_year:
            for month in range(self.sc_start_mon, self.sc_end_mon + 1):
                date.append(F'{self.ctc_year} {month}')
        else:
            for month in range(self.sc_start_mon, 13):
                date.append(F'{self.sc_start_year} {month}')
            for month in range(1, self.sc_end_mon + 1):
                date.append(F'{self.sc_end_year} {month}')
        for d in date:
            ds = d.split()
            for day in range(1, 32):
                try:
                    sc = SC(int(ds[0]), int(ds[1]), day)
                    sc_string = sc.to_sc()
                    if self.raw_date == sc_string:
                        self.date = [ds[0], ds[1], str(day)]
                        return ds[0] + '年' + ds[1] + '月' + str(day) + '日'
                except ValueError:
                    pass
        return None

    def get_year(self):
        if not self.date:
            if not self.find_sc():
                return '没有找到年'
        return self.date[0]

    def get_month(self):
        if not self.date:
            if not self.find_sc():
                return '没有找到月'
        return self.date[1]

    def get_day(self):
        if not self.date:
            if not self.find_sc():
                return '没有找到日'
        return self.date[2]


if __name__ == '__main__':
    c = CTC(ctc_year=2017, ctc_mon=-6, ctc_day=8)  # 农历的日期2017年闰6月初八
    print(c.find_sc())  # 阳历：2017年7月30日
    print(c.get_year())  # 2017
    print(c.get_month())  # 7
    print(c.get_day())  # 30



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/4 0004
# @Email : jtyoui@qq.com
# @Software : PyCharm
from jtyoui.data import Han_J_F  # 引入简体字与繁体字的映射表

J_F, F_J = Han_J_F, {f: j for j, f in Han_J_F.items()}  # 简体字:J表示, 繁体字用F表示


def j_to_f(str_):
    """
    简体字转繁体字
    :param str_: 字符串
    :return: 转换后的字符串生成器
    """
    for s in str_:
        yield J_F.get(s, s)


def f_to_j(str_):
    """
    繁体字转简体字
    :param str_: 字符串
    :return: 转换后的字符串生成器
    """
    for s in str_:
        yield F_J.get(s, s)


if __name__ == '__main__':
    j = j_to_f('千载正字一夕改,如今吾辈来重光!')
    print(''.join(j))
    f = f_to_j('千載正字一夕改,如今吾輩來重光!')
    print(''.join(f))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/13 10:55
# @Author: Jtyoui@qq.com
import numpy as np
from sklearn.preprocessing import LabelBinarizer
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neighbors import KNeighborsRegressor

"""
KNN分类算法例子
"""


def knn_class_fit(train, label):
    """训练数据模型"""
    binary = LabelBinarizer()  # 二值化
    y_ = binary.fit_transform(label)
    clf = KNeighborsClassifier()
    clf.fit(train, np.ravel(y_))
    return clf, binary


def knn_class_predict(models, predict_value):
    """预测模型"""
    predict = models[0].predict(predict_value)
    return models[1].inverse_transform(predict)


def knn_regress_fit(train, label):
    clf = KNeighborsRegressor()
    clf.fit(train, label)
    return clf


def knn_regress_predict(models, predict_value):
    return models.predict(predict_value)


if __name__ == '__main__':
    # 身高和体重去预测性别
    x = np.array([
        [156, 64],
        [170, 86],
        [183, 84],
        [191, 80],
        [155, 49],
        [163, 59],
        [180, 67],
        [158, 54],
        [170, 67]
    ])
    y = np.array(['male', 'male', 'male', 'male', 'female', 'female', 'female', 'female', 'female'])
    test = np.array([[155, 70]])

    model = knn_class_fit(x, y)
    print(knn_class_predict(model, test))  # 预测：['female']，正确：['female']

if __name__ == '__main__':
    # 利用身高和性别来预测体重
    x = np.array([
        [158, 1],
        [170, 1],
        [183, 1],
        [191, 1],
        [155, 0],
        [163, 0],
        [180, 0],
        [158, 0],
        [170, 0],
        [168, 1],
        [180, 1],
        [160, 0]
    ])
    y = np.array([64, 86, 84, 80, 49, 59, 67, 54, 67, 65, 96, 52])
    test = np.array([[169, 0]])

    model = knn_regress_fit(x, y)
    print(knn_regress_predict(model, test))  # 预测：[65.8]，正确：[67]



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/24 21:44
# @Email  : jtyoui@qq.com
# @Software: PyCharm
"""声明：该函数是爬取造句网（http://zaojv.com/）句子，仅仅用于学习交流。"""
import requests
from jtyoui.web import headers_ua
import re


class Sentence:
    """造句函数，输入一个词语，造出一些句子
    >>>s = Sentence()
    >>>data = s['我爱你']
    >>>>for jz in data:print(jz)
    """

    def __init__(self):
        self.url = 'http://zaojv.com/wordQueryDo.php'
        from bs4 import BeautifulSoup  # 安装bs4: pip install bs4
        self._BeautifulSoup = BeautifulSoup
        self.total = 1
        self.page = 1

    def _not_included_data(self, word):
        """未录取句子提取"""
        data_ls = []
        response = requests.post(self.url, headers=headers_ua(), data={'wo': word, 'directGo': 1, 'nsid': 0})
        text = response.text
        soup = self._BeautifulSoup(text, features="lxml")
        ls = soup.find('div', id='content').contents
        for d in ls:
            if not isinstance(d, str) and ('function' not in d.text):
                t = d.text.replace('\xa0', '').strip()
                if '造句' in t:
                    t = re.sub('[(（\\[【].*[^造]*造句.*[)】\\]）]', '', t)
                data_ls.append(t)
        return data_ls

    def _analysis_data(self, text):
        """录用句子提取"""
        data_ls = []
        soup = self._BeautifulSoup(text, features="lxml")
        page = soup.find('div', style='text-align:center;margin-top:10px;').text.strip().split('\xa0')
        self.page, self.total = page[7].split(r'/')
        ls = soup.find('div', id='all').contents
        for d in ls:
            if not isinstance(d, str) and ('function' not in d.text):
                t = d.text.replace('\xa0', '').strip()
                if '造句' in t:
                    t = re.sub('[(（\\[【].*[^造]*造句.*[)】\\]）]', '', t)
                data_ls.append(t)
        return data_ls

    def _redirect_link(self, word):
        """获得重定向的链接"""
        response = requests.post(self.url, headers=headers_ua(), data={'wo': word, 'directGo': 1, 'nsid': 0})
        return response.url

    @staticmethod
    def _get_page_data(url):
        """获得链接中的数据"""
        response = requests.get(url, headers=headers_ua())
        return response.text

    def make_sentence(self, word):
        """制造句子
        :param word:输入一个词语:比如:刘德华、万绮雯、宇宙等
        """
        data_ls = []
        url_ = self._redirect_link(word)
        if url_ == self.url:
            d = self._not_included_data(word)
        else:
            text = self._get_page_data(url_)
            d = self._analysis_data(text)
        data_ls.extend(d)
        while self.total != self.page:
            page = int(self.page) + 1
            u = url_[:-5] + '_' + str(page) + '.html'
            text = self._get_page_data(u)
            data_ls.extend(self._analysis_data(text))
        return data_ls

    def __getitem__(self, item):
        return self.make_sentence(item)


if __name__ == '__main__':
    s = Sentence()
    data = s['我爱你']
    for jz in data:
        print(jz)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/14 9:15
# @Author: Jtyoui
import requests
from random import choice
from jtyoui.error import BrowserTypeError, UAPageVersionError
import pickle
import os

browsers = None


def ua(browser='chrome', version='0.1.11', update=False):
    """
    随机返回browser浏览器一条UA
    :param browser: 浏览器名字,包括如下:[chrome,opera,firefox,internetexplorer,safari]
    :param version: 爬取ua网页版本
    :param update:是否更新保存的UA文件数据
    :return: 字典类型
    """
    global browsers
    save_path = r'C:\browser.json' if os.name == 'nt' else '/etc/browser.json'
    other_save_path = r'D:\browser.json' if os.name == 'nt' else '/home/browser.json'
    browser_type = ['chrome', 'opera', 'firefox', 'internetexplorer', 'safari']
    b = {
        'c': 'chrome',
        'o': 'opera',
        'ff': 'firefox',
        'ie': 'internetexplorer',
        's': 'safari'
    }
    browser = b.get(browser, browser)
    if browser not in browser_type:
        raise BrowserTypeError('浏览器类型异常,类型支持:[chrome,opera,firefox,internetexplorer,safari]')
    if browsers:
        return choice(browsers)
    if (not update) and os.path.exists(save_path):
        with open(save_path, 'rb')as fp:
            json = pickle.load(fp)
    elif (not update) and os.path.exists(other_save_path):
        with open(other_save_path, 'rb')as fp:
            json = pickle.load(fp)
    else:
        print(f'第一次在线加载，加载保存的文件在：{save_path}，以后都是离线加载，想改变请修改参数:update==True')
        cache = 'https://fake-useragent.herokuapp.com/browsers/' + version
        response = requests.get(cache)
        if response.status_code != 200:
            if version != '0.1.3':
                return ua(browser, '0.1.3')
            else:
                raise UAPageVersionError('版本问题,重新指定版本')
        json = response.json()
        try:
            with open(save_path, 'wb')as fp:
                pickle.dump(json, fp)
        except PermissionError:
            with open(other_save_path, 'wb')as fp:
                pickle.dump(json, fp)
    browsers = json['browsers'][browser]
    return choice(browsers)


random = ua


def headers_ua():
    return {'user-agent': random()}


if __name__ == '__main__':
    print(random())
    print(headers_ua())
    print(ua(update=True))  # 更新文件



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/29 9:11
# @Author: Jtyoui@qq.com
import functools
import logging
from jtyoui.logger.Log import set_log_file_config, get_log_config
import os


def log(log_file, config_file=None):
    """日志装饰器，用于测试函数的时候打印日志。
    :param log_file:日志地址保存地方
    :param config_file:日志配置地址保存地址。关于日志配置：可以参考官网配置： https://docs.python.org/3.7/library/logging.config.html
    """

    def inner(fun):
        set_log_file_config(get_log_config(config_path=config_file, custom_dir=log_file))

        @functools.wraps(fun)
        def wraps(*args, **kwargs):
            try:
                logging.getLogger('info').info(f'正在执行：{fun.__name__}函数')
                f = fun(*args, **kwargs)
                logging.getLogger('info').info(f'执行完毕：{fun.__name__}函数')
                return f
            except Exception as e:
                logging.getLogger('info').error(f'执行：{fun.__name__}函数异常，异常信息:{str(e)}')
                with open(log_file + os.sep + 'error.log', 'a', newline='\n')as f:
                    f.write('+' * 70 + os.linesep)
                logging.getLogger('error').exception(e)
                with open(log_file + os.sep + 'error.log', 'a', newline='\n')as f:
                    f.write('#' * 70 + os.linesep + os.linesep)
                raise e

        return wraps

    return inner


if __name__ == '__main__':
    @log('./ger', None)
    def division():
        s = 1 / 0
        return s


    division()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/19
# @Email : jtyoui@qq.com
from .header import header, free_header  # 封装header
from .ua import random, ua, headers_ua  # 封装UA
from .HTML import ParseHtml  # 增加HTML解析
from .Request import get, post, get_js  # 增加爬虫网站
from .interfaces import *  # 增加后端调用接口测试



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/1 15:53
# @Author: Jtyoui@qq.com
import copy


def get_vocab(path, tags):
    data, label, vocab = [], [], {}
    # <PAD>：0, <UNK>：1, <END>：2
    i = 3
    with open(path, encoding='utf-8') as fp:
        ds, ls = [], []
        for line in fp:
            line = line.strip()
            if line:
                k, v = line.split()
                if k not in vocab:
                    vocab[k] = i
                    i += 1
                ds.append(vocab[k])
                ls.append([tags[v]])
            else:
                data.append(copy.deepcopy(ds + [2]))
                label.append(copy.deepcopy(ls + [2]))
                ds, ls = [], []

    return data, label, vocab


if __name__ == '__main__':
    tag = {'O': 0,
           'B-a': 1, 'I-a': 2,
           'B-b': 3, 'I-b': 4,
           'B-c': 5, 'I-c': 6
           }
    train, test, vocabs = get_vocab(r'train.txt', tag)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/23 16:33
# @Author: Jtyoui@qq.com


class BayesProbability:
    """贝叶斯"""

    def __init__(self):
        self.name_prob = dict()
        self.name_prior_prob = dict()

    def __setitem__(self, name, prob):
        """
        初始化name的概率
        :param name: 名字
        :param prob: 独立概率
        """
        self.name_prob.setdefault(name, prob)

    def prior_probability(self, name, prob):
        """先验概率
        :param name: 名字
        :param prob: 先验概率
        """
        self.name_prior_prob[name] = prob

    def posterior_probability(self, name):
        """后验概率"""
        h = 0
        for _, prob in self.name_prior_prob.items():
            h += prob
        h /= len(self.name_prior_prob)
        return (self.name_prob[name] * self.name_prior_prob[name]) / h


if __name__ == '__main__':
    bp = BayesProbability()
    bp['bowl_one'] = 1 / 2
    bp['bowl_two'] = 1 / 2
    bp.prior_probability('bowl_one', 3 / 4)
    bp.prior_probability('bowl_two', 1 / 2)
    print(bp.posterior_probability('bowl_one'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/11/16 19:35:00
# @Email : jtyoui@qq.com
# @Software : PyCharm
from jtyoui.web import get
import re


def double_data_chart(start=None, end=None):
    """爬取双色球数据,第一列数据是信息头。
    :param start:开始期号：默认是第一期时间。
    :param end:结束期号：默认是现在时间。
    :return:二维列表。
    """
    if start is None and end is None:
        return double_data_chart(*_get_start_end())
    header = ['期号', '红球1', '红球2', '红球3', '红球4', '红球5', '红球6', '篮球', '奖池',
              '一等奖注数', '一等奖奖金', '二等奖注数', '二等奖奖金', '总投注额', '开奖日期']
    ls = [header]
    url = f'https://datachart.500.com/ssq/history/newinc/history.php?start={start}&end={end}'
    data = get(url)
    response = data.content.decode('utf-8')
    for charts in re.findall(r'<tr class="t_tr1">.+?</tr>', response):
        td = re.findall('<td.*?>(.+?)</td>', charts)
        td.pop(9)
        m = map(lambda x: x if '-' in x else int(x.replace(',', '')), td[1:])
        ls.append(list(m))
    return ls


def _get_start_end():
    """获取开始期号和结束期号"""
    url = 'https://datachart.500.com/ssq/history/history.shtml'
    data = get(url)
    response = data.content.decode('gbk')
    search = re.search('<input id="end" name="end" value="(.+?)" size="10" />', response)
    start, end = search.start() + 34, search.end() - 14
    return 3001, response[start:end]


if __name__ == '__main__':
    print(double_data_chart())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2018/2/28 10:58
# @Email  : jtyoui@qq.com
# @Software: PyCharm


import math
import re
from threading import Thread
import queue

ALL_WORDS = dict()
All_LENS = 0


class Neologism(Thread):
    def __init__(self, q, split_num=4):
        Thread.__init__(self)
        self.queue = q
        self.split_num = split_num

    def run(self):
        while True:
            try:
                line = self.queue.get_nowait()
                self.read_string(line)
                self.queue.task_done()
            except queue.Empty:
                return

    def read_string(self, st, split_seq='[，。！？：]'):
        """
        讲字符按照split_seq格式来分割
        :param st: 字符串
        :param split_seq: 字符分割
        :return: None
        """
        ls = re.split(split_seq, st)
        self.read_ls(ls=ls)

    def read_ls(self, ls):
        """数据类型[str]
        :param ls: 表示链表
        """
        global All_LENS
        for word in ls:
            All_LENS += len(word)
            clean_data, lens = clean(data=word)
            if lens > 2:
                self.split(clean_data, lens)

    def split(self, words, lens):
        """
        拆分字符，最大匹配num个字符，并也字典的形式返回，
        [出现次数,出现频率,凝固程度,自由程度,关键字的左邻,关键字的右邻](作为信息熵的衡量)
        """
        global ALL_WORDS
        for i in range(0, lens):
            for j in range(1, self.split_num + 1):
                if i + j < lens:
                    key = words[i:i + j]
                    word = ALL_WORDS.get(key)
                    if word:
                        word[0] += 1
                        word[4].append(words[i - 1])
                        word[5].append(words[i + j])
                    else:
                        ALL_WORDS[key] = [1, 0.0, 1, 0, [words[i - 1]], [words[i + j]]]


def statistics(key_list):  # 统计每个单词的频率
    for key in key_list:
        ALL_WORDS[key][1] = ALL_WORDS[key][0] / All_LENS


def handle(key_list):
    """
    处理数据
    计算左邻字集合和右邻字集合的频率，左邻字信息熵和右邻字信息熵中的较小值
    计算凝固程度,自由程度
    """
    for key in key_list:
        word_list = ALL_WORDS[key]  # 获得一个单词的链表信息
        if len(key) == 1:
            continue
        end_all = front_all = 0.0
        left = word_list[1] / (ALL_WORDS[key[0]][1] * ALL_WORDS[key[1:]][1])  # 左邻字集合的频率
        right = word_list[1] / (ALL_WORDS[key[-1]][1] * ALL_WORDS[key[:-1]][1])  # 右邻字集合的频率

        for front in word_list[4]:
            if ALL_WORDS.get(front):
                front_all -= math.log(ALL_WORDS[front][1]) * ALL_WORDS[front][1]  # 左邻字的信息熵

        for end in word_list[5]:
            if ALL_WORDS.get(end):
                end_all -= math.log(ALL_WORDS[end][1]) * ALL_WORDS[end][1]  # 右邻字的信息熵

        # 左邻字集合和右邻字集合的频率相比较.谁越少说明该词语越容易接近谁
        word_list[2] = left if left < right else right

        # 左邻字集合的信息熵和右邻字集合的信息熵的相比较.谁的信息熵越少说明该集合提供的信息越大
        word_list[3] = front_all if front_all < end_all else end_all


def filter_words(frequency, cond, free, flag):
    """
    过滤一些不重要的数据
    [出现次数,出现频率,凝固程度,自由程度]
    :param frequency: 过滤的频率
    :param cond:过滤凝聚度
    :param free:过滤自由度
    :param flag: 是否是并且还是或者,默认是或者，满足一个就过滤
    :return:过滤后的数据字典
    """
    key_words = dict()
    for key in ALL_WORDS.keys():
        if len(key) <= 1:
            continue
        one_word = ALL_WORDS[key]
        if flag:
            if one_word[1] > frequency and one_word[2] > cond and one_word[3] > free:
                key_words[key] = [one_word[0], one_word[1], one_word[2], one_word[3]]
        else:
            if one_word[1] > frequency or one_word[2] > cond or one_word[3] > free:
                key_words[key] = [one_word[0], one_word[1], one_word[2], one_word[3]]
    return key_words


def read_file(file, file_encoding='utf-8'):
    """
    读取文件内容，注意文件是UTF-8的格式且不是BOM格式
    :param file: 读取的文件
    :param file_encoding: 文本编码
    """
    queues = queue.Queue(maxsize=0)
    with open(file, encoding=file_encoding) as fp:
        for line in fp:
            queues.put(line)
    return queues


def clean(data):
    # 去除非中文字符
    words = [work for work in data if 19968 < ord(work) < 40959]
    return ''.join(words), len(words)


def thread_analysis(file, thread_num=10, split_num=4, frequency=0.0001, cond=10, free=0.1, flag=False):
    """
    :param file: 训练的文本
    :param thread_num: 线程数
    :param split_num: 匹配个数
    :param frequency: 频率
    :param cond: 凝聚度
    :param free: 自由度
    :param flag:是否是并且还是或者,默认是或者，满足一个就过滤
    :return: 分析完毕的字典
    """
    queues = read_file(file)
    neologisms = [Neologism(split_num=split_num, q=queues) for _ in range(thread_num)]
    for neologism in neologisms:
        neologism.start()
    queues.join()
    keys_list = list(ALL_WORDS.keys())
    size = len(keys_list) // split_num + 1
    print("开始统计频率.........")
    thread_open(split_num, statistics, keys_list, size)
    print("开始处理数据.........")
    thread_open(split_num, handle, keys_list, size)
    print("开始过滤数据.........")
    return filter_words(frequency, cond, free, flag)


def thread_open(split_num, target, keys_list, size):
    """
    开启多线程
    :param split_num: 线程数
    :param target: 被开启的方法
    :param keys_list: 所有单词的键链表
    :param size: 被分割成一块的大小
    :return: 无
    """
    threads = []
    for i in range(split_num):
        t = Thread(target=target, args=(keys_list[i * size:(i + 1) * size],))
        threads.append(t)
        t.start()
    for t in threads:
        t.join()


if __name__ == '__main__':
    neologism_words = thread_analysis(file='小时代.txt', thread_num=10, frequency=0.00001, split_num=8, cond=100,
                                      flag=True)
    for k, v in neologism_words.items():
        print('key:{0} count:{1} frequency:{2} cond:{3} free:{4}'.format(k, v[0], v[1], v[2], v[3]))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/5 17:41
# @Author: Jtyoui@qq.com
import os


def hide_file(file_path):
    """隐藏文件夹或者文件"""
    x = os.system(f'attrib +a +s +h +r {file_path}..')
    if x == 0:
        return True
    else:
        return False


def display_file(file_path):
    """显示文件夹或者文件"""
    x = os.system(f'attrib -a -s -h -r {file_path}..')
    if x == 0:
        return True
    else:
        return False


if __name__ == '__main__':
    print(hide_file(r'D:/1/2.txt'))
    print(display_file(r'D:/1/2.txt'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/21 15:04
# @Author: Jtyoui@qq.com
from jtyoui.decorator import parameter_set_length
from jtyoui.statistics.analysis import AnalysisMath, cov

"""
皮尔森相关性系数
Pearson相关系数是用来衡量两个数据集合是否在一条线上面，它用来衡量定距变量间的线性关系。
相关系数 0.8-1.0 极强相关
0.6-0.8 强相关
0.4-0.6 中等程度相关
0.2-0.4 弱相关
0.0-0.2 极弱相关或无相关
就好比我们想研究人跑步的速度与心脏跳动的相关性，如果你无论跑多快，
心跳都不变（即心跳这个变量的标准差为0），或者你心跳忽快忽慢的，
却一直保持一个速度在跑（即跑步速度这个变量的标准差为0），
那我们都无法通过皮尔森相关性系数的计算来判断心跳与跑步速度到底相不相关。
"""


@parameter_set_length
def pearson_coefficient(sample_x, sample_y):
    """
    皮尔森相关性系数
    :param sample_x: x样本数据变量集合
    :param sample_y: y样本数据变量集合
    :return:x和y的相关系数
    """
    ana = AnalysisMath()  # 数学分析模块
    numerator = cov(sample_x, sample_y)  # 分子是协方差
    denominator = pow(ana.variance(sample_x) * ana.variance(sample_y), 0.5)  # 分母是两个方差的积开平方
    return numerator / denominator


if __name__ == '__main__':
    print(pearson_coefficient([1, 2, 3, 4, 5, 6], [0.3, 0.9, 2.7, 2, 3.5, 5]))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2018/2/2 10:58
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.bs import binary_system  # 任意进制转换
from jtyoui.code import cr  # 二维码识别
from jtyoui.plunar import *  # 农历、阳历
from jtyoui.sogou import SoGou  # 下载搜狗
from jtyoui.word import *  # 新词发现
from jtyoui.mail import *  # 邮箱
from jtyoui.data import *  # 常量
from jtyoui.fractal import *  # 分形
from jtyoui.web import *  # 网页
from jtyoui.language import *  # 语言
from jtyoui.regular import *  # 正则
from jtyoui.decorator import *  # 装饰器
from jtyoui.error import *  # 异常
from jtyoui.maps import *  # 地图
from jtyoui.statistics import *  # 文本统计方法
from jtyoui.bayes import *  # 贝叶斯算法
from jtyoui.tools import *  # 常见的工具类函数
from jtyoui.c import *  # 增加c代码引用
from jtyoui.baidu import *  # 增加关于百度的资料及应用
from jtyoui.jp import *  # 提取日语意思
from jtyoui.compress import *  # 压缩包文件
from jtyoui.algorithm import *  # 常见算法
from jtyoui.cmd import *  # CMD命令
from jtyoui.interface import *  # 界面
from jtyoui.reflex import *  # 反射
from jtyoui.logger import *  # 增加日志文件夹
from jtyoui.project import *  # 项目接口
from jtyoui.person import *  # 中国人属性
from jtyoui.time import *  # 关于处理时间模块
from jtyoui.reptile import *  # 关于爬虫的模块
from jtyoui.ee import *  # 抽取实体
from jtyoui.flasks import *  # 对flask框架的补充

__all__ = [a for a in dir() if not a.startswith('_')]
__all__.extend(['game', 'imagepdf', 'wx', 'neuralNetwork', 'ml'])  # 游戏 pdf和照片互转 微信抓电影和聊天机器人
__version__ = f'{NOW_TIME.tm_year}.{NOW_TIME.tm_mon:0>2}.{NOW_TIME.tm_mday:0>2}'
__author__ = 'Jtyoui'
__description__ = f'这是一个Python集合包,包含了:{len(__all__)}个模块。'
__email__ = 'jtyoui@qq.com'
__web__ = 'https://blog.jtyoui.com'
__blog__ = 'https://blog.csdn.net/qq_23985359'



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/10 0010
# @Email : jtyoui@qq.com
# @Software : PyCharm
from .gitee import fetch_gitee, download_gitee  # 爬取我自己Gitee上的数据
from .han_table import Han_J_F  # 导入繁体字和简体字的数据
from .constant import *  # 平时常见的常量
from .province import *  # 中国省市所有名字
from .Train import find_train_info, find_train_desc, find_train_desc_info, Train_Station  # 关于火车站的信息
from .Enums import *  # 增加枚举
from .Address import *  # 查询地址
from .TimeZone import TZ  # 时区
from .WeatherAddress import WeatherForecast  # 天气预报
from .methods import *  # 一些常见的方法
from .wordNature import *  # 词性
from .fileFormat import FileFormat  # 文件格式
from .genericity import fun_generic  # 重载
from .phoneRegularity import *  # 电话号码匹配
from .CardId import *  # 增加身份证校验功能
from .platenumber import *  # 增加车牌号验证




#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/15  17:27
# @Author: jtyoui@qq.com
from jtyoui.file_zip import file_zip_path, sep


def word_nature():
    """读取file_zip下的word_nature文件，返回词性字典"""
    p = file_zip_path + sep + 'word_nature.txt'
    cx = reader_conf(p)
    return cx


def reader_conf(path: str, encoding: str = 'UTF-8') -> dict:
    """读取配置文件
    [capitalize]
    a
    b
    :param path: 配置文件路径
    :param encoding: 文件编码
    """
    cx = {}
    with open(path, encoding=encoding)as fp:
        for data in fp:
            data = data.strip()
            if data.startswith('[') and data.endswith(']'):
                key = data[1:-1]
                cx.setdefault(key, [])
            elif (not data.startswith('#')) and data:
                cx[key].append(data)
            else:
                pass
    return cx


def save_conf(cx: dict, path: str, encoding='UTF-8'):
    """保存配置文件
    :param cx: 保存的字典类型：{str：list}
    :param path: 保存的路径
    :param encoding: 保存文件的编码
    """
    with open(path, 'w', encoding=encoding, newline='\n')as fp:
        for key, value in cx.items():
            fp.write(f'[{key}]\n')
            if isinstance(value, (list, tuple)):
                for v in value:
                    fp.write(v.strip() + '\n')
            else:
                raise TypeError('字典的值必须是列表或者元组类型')


if __name__ == '__main__':
    print(word_nature())



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/2 0002
# @Email : jtyoui@qq.com
# @Software : PyCharm
import requests
from jtyoui.web import random
from jtyoui.error import LibraryNotInstallError
import json

try:
    import itchat  # 安装 pip install itchat
except ModuleNotFoundError:
    raise LibraryNotInstallError("安装 pip install itchat")
import os

# 全局请求头
files, headers, body, timeouts, resHeader = {}, {}, {}, {}, {}
app_id, app_secret = '86760', 'ccece9bc703d4064b95f31ced8f84c42'
rec_tmp_dir = os.path.join(os.getcwd(), 'tmp/')
oneself_name = show_api = None
one_run, run = False, True


class ShowApiRequest:
    def __init__(self, url, my_app_id, my_app_secret):
        if not os.path.exists(rec_tmp_dir):
            os.mkdir(rec_tmp_dir)
        self.url = url
        self.my_appId = my_app_id
        self.my_appSecret = my_app_secret
        body["showapi_appid"] = my_app_id
        body["showapi_sign"] = my_app_secret
        headers["User-Agent"] = random()

    @staticmethod
    def add_file(key, value_url):
        with open("{}".format(value_url), 'rb')as f:
            files[key] = f.read()

    @staticmethod
    def add_body(key, value):
        body[key] = value

    # 设置连接时间和读取时间
    @staticmethod
    def set_timeout(connect_timout, read_timeout):
        timeouts["connect_timout"] = connect_timout
        timeouts["read_timeout"] = read_timeout

    def post(self):
        if not timeouts:
            res = requests.post(self.url, files=files, data=body, headers=headers)
        else:
            timeout = (timeouts["connect_timout"], timeouts["read_timeout"])
            res = requests.post(self.url, files=files, data=body, headers=headers, timeout=timeout)
        return res


def show_robot(info, nick_name, img_dir):  # 调用图灵机器人的接口
    show_api.add_body('userid', nick_name)
    if info:
        show_api.add_body('info', info)
    if img_dir:
        show_api.add_file('img', img_dir)
    res = show_api.post()
    data = json.loads(res.text)
    text = data['showapi_res_body'].get('text')
    return data, text


def public_message(msg):  # 返回公用信息
    global run, one_run
    send_name = msg['FromUserName']
    if msg.text == '禁止回复':  # 对于其他人调用
        run = False
    if msg.text == '启动回复':
        run = True

    if send_name == oneself_name:
        if msg.text == '启动回复':  # 对于我来调用
            run = one_run = True
        if msg.text == '禁止回复':
            one_run = False
        if not one_run:
            return 0, 0
    if run:
        nick_name = msg['User']['NickName']
        remark_name = msg['User']['RemarkName']
        return send_name, nick_name or remark_name
    return 0, 0


@itchat.msg_register(itchat.content.TEXT)  # 获得文本信息
def text_reply(msg):
    send_name, call_name = public_message(msg)
    if send_name:
        _, text = show_robot(msg.text, call_name, None)
        if not text:
            text = '人工智障出现问题,等待主人抢修中!'
        text = '[自动回复]:' + text + '---------->>>[输入:禁止回复,就停止机器回复.输入:启动回复,就会启动机器回复]'
        itchat.send_msg(text, send_name)


@itchat.msg_register(itchat.content.PICTURE)  # 获得图片信息
def img_reply(msg):
    send_name, call_name = public_message(msg)
    if send_name:
        img_address = rec_tmp_dir + msg['FileName']
        msg.download(img_address)
        _, text = show_robot(None, call_name, img_address)
        if text:
            text = '[自动回复]:' + text
            itchat.send_msg(text, send_name)


def oneself(wx_name):  # 获得自己的信息
    global oneself_name
    one = itchat.search_friends(name=wx_name)
    oneself_name = one[0]['UserName']


def auto_start(wx_name):
    global show_api
    show_api = ShowApiRequest('http://route.showapi.com/60-27","my_appId', app_id, app_secret)
    itchat.auto_login(hotReload=True)
    oneself(wx_name)  # 输入自己的微信名
    itchat.run()


if __name__ == '__main__':
    auto_start(wx_name='Jtyoui')  # 输入自己的微信名,这个是自己取的名字



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/4 0004
# @Email : jtyoui@qq.com
# @Software : PyCharm

from .han import j_to_f, f_to_j  # 增加简体字互转繁体字
from .PinYin import load_pin_yin, chinese_to_pin_yin  # 增加拼音
from .Translates import translate_to_chinese  # 增加翻译到中文
from .CEC import ChineseError  # 汉语纠错
from .BaiDuTranslate import bai_du_translate, BaiDuLanguage  # 百度翻译



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/10 0010
# @Email : jtyoui@qq.com
# @Software : PyCharm
import os
import requests
from jtyoui.web import random
from urllib.request import urlretrieve

GITEE = 'https://gitee.com/tyoui/{project}/raw/master/{package}/{name}'  # gitee上保存数据的格式


def fetch_gitee(package, name, project='logo'):
    """
    爬取非结构文本数据
    :param package: 包名
    :param name: 文件名
    :param project: 项目名
    :return: 返回字节数据
    """
    url = GITEE.format(project=project, package=package, name=name)
    response = requests.get(url=url, headers={'User-Agent': random()})  # 动态增加UA
    content = response.content
    return content


def download_gitee(package, name, file_dir=None, project='logo'):
    """
    下载非结构文本数据
    :param package: 包名
    :param name: 文件名
    :param file_dir: 保存文件的文件夹地址
    :param project: 项目名
    :return: 下载成功返回'success',失败返回'fail'
    """
    if not file_dir:
        address = os.path.join(os.getcwd(), name)  # 默认保存地址是运行项目当前的位置
    elif os.path.isdir(file_dir):
        address = os.path.join(file_dir, name)
    elif not os.path.exists(file_dir):
        os.mkdir(file_dir)
        address = os.path.join(file_dir, name)
    else:
        address = file_dir
    url = GITEE.format(project=project, package=package, name=name)
    try:
        place = urlretrieve(url, address)  # 下载
        print('\033[1;33m' + place[0])
        return 'success'
    except Exception as e:
        print(e)
        return 'fail'


if __name__ == '__main__':
    status = download_gitee('logo', 'logo.png', file_dir='D://temp')  # 将照片logo.png下载到D盘
    print(status)
    pillow = fetch_gitee('logo', 'logo.png')  # 返回的是字节数据
    print(pillow)
    # byte = io.BytesIO(pillow)  # 转成字节流
    # Image.open(fp=byte)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/2 0002
# @Email : jtyoui@qq.com
# @Software : PyCharm
from .AutoChat import auto_start  # 自动聊天机器人
from .Movie import movie_start  # 自动爬取电影机器人



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/10 13:47
# @Author: Jtyoui@qq.com
from jtyoui.error import MathValueWarning
import math
import warnings


def theorem_Zero(function, x1: float, x2: float) -> float:
    """零点定理：定义一个函数：x^3-2x-5=0,求x等于多少。x的值域：[1,1000],
    原理利用二分法不断的逼近，求出答案。
    :param function:定一个函数
    :param x1:开始值
    :param x2:结束值
    :return:返回零点的值
    """
    if function(x1) == 0:
        return x1
    elif function(x2) == 0:
        return x2
    elif function(x1) * function(x2) > 0:
        warnings.warn('[a,b]区间的值应该满足:f(a)*f(b)<0', category=MathValueWarning)
        return math.inf
    else:
        mid = x1 + (x2 - x1) / 2.0
        while abs(x1 - mid) > math.pow(10, -9):  # x值小于10亿分之一
            if function(mid) == 0:
                return mid
            elif function(mid) * function(x1) < 0:
                x2 = mid
            else:
                x1 = mid
            mid = x1 + (x2 - x1) / 2.0
        return mid


if __name__ == '__main__':
    # 定义一个函数：x^3-2x-5=0,求x等于多少。x的值域：[1,1000]
    print(theorem_Zero(lambda x: x ** 2 - 1, 0, 1000))
